
/* External interface

Shadow Generation:
VertexShader output:
	NVSHADOWS, NOSHADOWS:
		nothing
	FLOATSHADOWS:
		float fDepth : TEXCOORDN;
VertexShader:
	NVSHADOWS, NOSHADOWS:
		nothing
	FLOATSHADOWS
		fDepth = TransformedPosition.z;
PixelShader:
	NVSHADOWS, NOSHADOWS:
		return tex2D(BaseSampler, TexCoords);	// if alpha test needed
		or return float (0,0,0,1);				// if not
	FLOATSHADOWS:
		clip(tex2D(BaseSampler, TexCoords).w - alpharef);	// if alpha test needed
		return depth;
	
Shadowed draw:
VertexShader output:
	float4 vShadowParams : TEXCOORDN;
VertexShader:
	vShadowParams = Draw_GetShadowParams(Position);
PixelShader:
	// if no specular and shade ambient + diffuse
	color = (ambient + diffuse) * Draw_GetShadowSimple(vShadowParams, DepthBias) * color;
	// otherwise
	float2 shade = DrawGetShadow(vShadowParams, DepthBias);
	color = (ambient + shade.y * diffuse) * diffusecolor + shade.x * specual * specularcolor;
*/


texture tShadowMap;
float4x4 matLightViewProj;
float4x4 matShadowTexProj;

sampler sShadowMapSampler = sampler_state
{
	texture = (tShadowMap);
	AddressU = clamp;
	AddressW = clamp;
	MipFilter = none;
#ifdef NVSHADOWS
	MipFilter = Linear;
	MagFilter = Linear;	
#endif
#ifdef FLOATSHADOWS
	MipFilter = Point;
	MagFilter = Point;
#endif
};

float4 Draw_GetShadowParams(float4 pos)
{
#ifdef NVSHADOWS
	return mul(pos, matShadowTexProj);
#endif
#ifdef FLOATSHADOWS
	float4 tmp = mul(pos, matShadowTexProj);
	tmp.w = mul(pos, matLightViewProj).z;
	return tmp;
#endif
#ifdef NOSHADOWS
	return 0;
#endif
}

float Draw_GetShadowSimple(float4 coords, float dbias)
{
#ifdef NVSHADOWS
	return tex2Dproj(sShadowMapSampler, coords).x * 0.4 + 0.6;
#endif
#ifdef FLOATSHADOWS
	return (coords.w - dbias - tex2D(sShadowMapSampler, coords.xy)) > 0 ? 0.6 : 1;
#endif	
#ifdef NOSHADOWS
	return 1;
#endif
}

float Draw_GetShadowValue(float4 coords, float dbias)
{
#ifdef NVSHADOWS
	return tex2Dproj(sShadowMapSampler, coords).x;
#endif
#ifdef FLOATSHADOWS
	return (coords.w - dbias - tex2D(sShadowMapSampler, coords.xy)) > 0 ? 0 : 1;
#endif	
#ifdef NOSHADOWS
	return 1;
#endif
}


// x = shade
// y = diffuse shade

float2 Draw_GetShadow(float4 coords, float dbias)
{
#ifdef NVSHADOWS
	float2 tmp;
	tmp.x = tex2Dproj(sShadowMapSampler, coords).x;
	tmp.y = 0.1 + 0.9 * tmp.x;
	return tmp;
#endif
#ifdef FLOATSHADOWS
	float2 tmp;
	tmp.x = (coords.w - dbias - tex2D(sShadowMapSampler, coords.xy)) > 0 ? 0 : 1;
	tmp.y = 0.1 + 0.9 * tmp.x;
	return tmp;
#endif
#ifdef NOSHADOWS
	return float2(1,1);
#endif
}


//EOF