// WarFog Effect Helper

float3 WarFogParams;	// x - InvMapSize, y - HalfMapSize, z - 1/fadewidth
texture WarFogTexture;

static const float kadd = 0.5;
static const float kmul = 1 - kadd;

sampler WarFogSampler = sampler_state
{
	Texture = (WarFogTexture);
	AddressU = Clamp;
	AddressV = Clamp;
	MinFilter = Point;
	MagFilter = Linear;
	MipFilter = None;
};

// GetWarFog - use planar mapping to calculate fog parameters
// input - 3D position
// return WarFog Coefficient, that attenuate colors
float3 GetWarFogCoords(float3 aPosition)
{
	float2 tmp = (WarFogParams.yy - abs(aPosition.xz)) * WarFogParams.z;
	return float3(aPosition.xz * WarFogParams.x + 0.5, min(tmp.x, tmp.y));
}

float3 GetWarFogCoords(float4 aPosition)
{
	float2 tmp = (WarFogParams.yy - abs(aPosition.xz)) * WarFogParams.z;
	return float3(aPosition.xz * WarFogParams.x + 0.5, min(tmp.x, tmp.y));
}


float GetWarFog(float3 aCoords)
{
	return (tex2D(WarFogSampler, aCoords).x * kmul + kadd) * saturate(aCoords.z);
}

float GetWarFog(float3 aCoords, float aMul, float aAdd)
{
	return (tex2D(WarFogSampler, aCoords).x * aMul + aAdd) * saturate(aCoords.z);
}