// bbskin.fx
// Effect for drawing skinned multipart model
// Parameters

//#define OLD_VERSION

#include "Inc/utils.fx"
#include "Inc/warfog.fx"
#include "Inc/shadows.fx"
#include "Inc/PointLight.fx"

#define iAlphaRefValue 0xA8
#define fAlphaRefValue 0.66

#ifdef GRAPH_DETAIL_LOW
	#define DISABLE_WIND_EFFECT
#endif

///////////////////////////////////////////////////////////////////////////////////
// PARAMETERS
///////////////////////////////////////////////////////////////////////////////////

// camera
float4x4	matWorld;			// 
float4x4	matViewProj;		//  view*proj
float3		vBillBrdUp;			// xyz - Billboard up vector
float4		vBillBrdRight;		// xyz - Billboard right vector, w - GameTime
// light
float3		vLightDir;			// xyz -  
float3		cLightDiffuse;		//   
float3		cLightAmbient;
// material
float3		cMatAmbient;			//   (*)
float3		cMatDiffuse;		//   
texture		tBaseTexture;		//  
// model
float4		cModelColor;		//   
float4		vParams;			// x - FrameOffset
float4x4	arrBones[36];		//   
float4		arrFrames[80];		//  
								// x - BoneId, y - FrameDamage, z - FrameVisible

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Point;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float	fPSize		:PSIZE;			//  
	float3	vNormal		:NORMAL;		//  
	float4	vColor		:COLOR0;		//  
	float2	vTex		:TEXCOORD0;		//  
	float2	vOffset		:TEXCOORD1;		//    
};

struct vsInputInstance
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float	fPSize		:PSIZE;			//  
	float3	vNormal		:NORMAL;		//  
	float4	vColor		:COLOR0;		//  
	float2	vTex		:TEXCOORD0;		//  
	float2	vOffset		:TEXCOORD1;		//    
	float4  vInstLine1	:TEXCOORD2;		// world matrix line 1
	float4  vInstLine2	:TEXCOORD3;		// world matrix line 2
	float4  vInstLine3	:TEXCOORD4;		// world matrix line 3
	float	vInstLine4	:TEXCOORD5;		// visibility
};


///////////////////////////////////////////////////////////////////////////////////
// SHADOW GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputShadow
{
	float4	vPos		:POSITION;		// 
	float2	vTex		:TEXCOORD0;		//  
#ifdef FLOATSHADOWS	
	float	fDepth		:TEXCOORD1;		//   
#endif
};

void ShadowGen(out vsOutputShadow vOut, vsInput vIn, float4x4 aTransform, float aVisible)
{
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), aTransform);
	vTPos.xyz += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight.xyz * vIn.vOffset.y) * vIn.fPSize;
	// compute resulting position
	vOut.vPos=mul(vTPos, matLightViewProj) * aVisible;
#ifdef FLOATSHADOWS	
	vOut.fDepth = vOut.vPos.z;
#endif	
	vOut.vTex = vIn.vTex;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(vBillBrdRight.w + vIn.vPos.x + vIn.vPos.z) * 0.002 * vIn.vPos.y;
#endif
}

vsOutputShadow vsShadow(vsInput vIn)
{
	vsOutputShadow vOut;
	int aFrame = ToIndex(vIn.vPos.w - vParams.x);
	float4 vFrameParams = arrFrames[aFrame];
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);
	vTPos = mul(vTPos, matWorld);
	vTPos.xyz += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight.xyz * vIn.vOffset.y) * vIn.fPSize;
	// compute resulting position
	vOut.vPos=mul(vTPos, matLightViewProj) * vFrameParams.z;
#ifdef FLOATSHADOWS	
	vOut.fDepth = vOut.vPos.z;
#endif	
	vOut.vTex = vIn.vTex;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(vBillBrdRight.w + vIn.vPos.x + vIn.vPos.z) * 0.002 * vIn.vPos.y;
#endif
	return vOut;
}

float4 psShadow(vsOutputShadow vIn): COLOR0
{
	clip(tex2D(sBaseSampler, vIn.vTex).w - fAlphaRefValue);
#ifdef FLOATSHADOWS
	return vIn.fDepth;	
#else
	return float4(0,0,0,1);
#endif
}

technique TechShadow
{
	pass pass1
	{
		CullMode = CCW;
		FillMode = Solid;
		ZEnable = true;
		ZWriteEnable = true;
		AlphaBlendEnable = false;
		AlphaTestEnable = false;
		AlphaFunc = GreaterEqual;
		StencilEnable = false;
		PointSpriteEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader = compile vs_2_0 vsShadow();
		PixelShader = compile ps_2_0 psShadow();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// DEFER LIGHT GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsBBDeferLightOutput
{
	float4	vPos		:POSITION;		// 
	float4  vTPos		:TEXCOORD0;
	float3  vTNormal	:TEXCOORD1;
	float2	vTex		:TEXCOORD2;		//  
};

void DrawDeferLight(out vsBBDeferLightOutput vOut, vsInput vIn, float4x4 aTransform, float aVisible)
{
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), aTransform);
	vTPos.xyz += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight.xyz * vIn.vOffset.y) * vIn.fPSize;
	// compute resulting position
	vOut.vPos=mul(vTPos, matViewProj) * aVisible;
	vOut.vTPos = vOut.vPos;
	vOut.vTNormal = vIn.vNormal;
	vOut.vTex = vIn.vTex;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(vBillBrdRight.w + vIn.vPos.x + vIn.vPos.z) * 0.002 * vIn.vPos.y;
#endif
}

vsBBDeferLightOutput vsDeferLight(vsInput vIn)
{
	vsBBDeferLightOutput vOut;
	int aFrame = ToIndex(vIn.vPos.w - vParams.x);
	float4 vFrameParams = arrFrames[aFrame];
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);
	vTPos = mul(vTPos, matWorld);
	vTPos.xyz += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight.xyz * vIn.vOffset.y) * vIn.fPSize;
	// compute resulting position
	vOut.vPos=mul(vTPos, matViewProj) * vFrameParams.z;
	vOut.vTex = vIn.vTex;
	vOut.vTPos =vOut.vPos;
	vOut.vTNormal = vIn.vNormal;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(vBillBrdRight.w + vIn.vPos.x + vIn.vPos.z) * 0.002 * vIn.vPos.y;
#endif
	return vOut;
}

float4 psDeferLight(vsBBDeferLightOutput vIn): COLOR0
{
	clip(tex2D(sBaseSampler, vIn.vTex).w - fAlphaRefValue);
	return CalcDeferLightPSColor(vIn.vTPos, vIn.vTNormal);
}

TECH_DEFER_LIGHT_BEGIN
	VertexShader = compile vs_2_0 vsDeferLight();
	PixelShader = compile ps_2_0 psDeferLight();
TECH_DEFER_LIGHT_END
	
///////////////////////////////////////////////////////////////////////////////////
// RENDER WITH SHADOW SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputDraw
{
	float4 vPos			:POSITION;		// 
	float2 vTex			:TEXCOORD0;		//  
	float3 vDiffuse		:COLOR0;		//  
	float3 vAmbient		:COLOR1;		// ambient color
	float3 vFogPos		:TEXCOORD1;
};

void Draw(out vsOutputDraw vOut, vsInput vIn, float4x4 aTransform, float aVisible)
{
	// compute transformed position
	float3 vTPos = mul(float4(vIn.vPos.xyz, 1), aTransform);
	vTPos += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight.xyz * vIn.vOffset.y) * vIn.fPSize;
	float3 vTNormal = mul(vIn.vNormal, (float3x3)aTransform);			//   
#ifdef NORMALIZE_NORMAL
	vTNormal = normalize(vTNormal);
#endif	
	float fDiffuseCoeff = dot(vTNormal, -vLightDir);				//    
	fDiffuseCoeff = fDiffuseCoeff * fDiffuseCoeff * fDiffuseCoeff * 2;
	vOut.vAmbient = cMatAmbient * cLightAmbient;
	// compute resulting position
	vOut.vPos = mul(float4(vTPos, 1), matViewProj) * aVisible;	
    vOut.vDiffuse = cMatDiffuse * cLightDiffuse * fDiffuseCoeff;
	vOut.vTex = vIn.vTex;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(vBillBrdRight.w + vIn.vPos.x + vIn.vPos.z) * 0.002 * vIn.vPos.y;
#endif
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
}

vsOutputDraw vsDraw(vsInput vIn)
{
	vsOutputDraw vOut;
	int aFrame = ToIndex(vIn.vPos.w - vParams.x);
	float4 vFrameParams = arrFrames[aFrame];
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);
	vTPos = mul(vTPos, matWorld);
	vTPos.xyz += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight.xyz * vIn.vOffset.y) * vIn.fPSize;
	float3 vTNormal = mul(vIn.vNormal, (float3x3)arrBones[ToIndex(vFrameParams.x)]);			//   
	vTNormal = mul(vTNormal, (float3x3)matWorld);
#ifdef NORMALIZE_NORMAL
	vTNormal = normalize(vTNormal);
#endif	
	float fDiffuseCoeff = dot(vTNormal, -vLightDir);				//    
	fDiffuseCoeff = fDiffuseCoeff * fDiffuseCoeff * fDiffuseCoeff * 2;
	vOut.vAmbient = cMatAmbient * cLightAmbient;
	// compute resulting position
	vOut.vPos = mul(vTPos, matViewProj) * vFrameParams.z;	
    vOut.vDiffuse = cMatDiffuse * cLightDiffuse * fDiffuseCoeff;
	vOut.vTex = vIn.vTex;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(vBillBrdRight.w + vIn.vPos.x + vIn.vPos.z) * 0.002 * vIn.vPos.y;
#endif
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
    return vOut;    	
}

float4 psDraw(vsOutputDraw vIn):COLOR0
{
	float4 vColor = tex2D(sBaseSampler, vIn.vTex);
	return float4(vColor.xyz * (vIn.vAmbient + vIn.vDiffuse) * GetWarFog(vIn.vFogPos), vColor.w);
}

technique TechDraw
{
	pass pass1
	{
		AlphaBlendEnable = false;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		AlphaTestEnable = true;
		AlphaRef = iAlphaRefValue;
		AlphaFunc = GreaterEqual;
		PointSpriteEnable = false;
		CullMode = CCW;
		FillMode = Solid;
		StencilEnable = false;
		ZEnable = true;
		ZWriteEnable = true;
		MultiSampleAntiAlias = false;
		VertexShader = compile vs_2_0 vsDraw();
		PixelShader = compile ps_2_0 psDraw();
	}
}

//////////////////////////////////////////////////////////////////////////////////
// Instancing
//////////////////////////////////////////////////////////////////////////////////

vsOutputShadow vsShadowInst(vsInputInstance vIn)
{
	vsOutputShadow vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2, 
		vIn.vInstLine3,
		0,0,0,1);
	ShadowGen(vOut, (vsInput)vIn, transpose(mFWorld), vIn.vInstLine4.x);
	return vOut;
}

vsOutputDraw vsDrawInst(vsInputInstance vIn)
{
	vsOutputDraw vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2, 
		vIn.vInstLine3,
		0,0,0,1);
	Draw(vOut, (vsInput)vIn, transpose(mFWorld), vIn.vInstLine4.x);
	return vOut;	
}

vsBBDeferLightOutput vsDeferLight1(vsInputInstance vIn)
{
	vsBBDeferLightOutput vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2, 
		vIn.vInstLine3,
		0,0,0,1);
	DrawDeferLight(vOut, (vsInput)vIn, transpose(mFWorld), vIn.vInstLine4.x);
	return vOut;
}

technique TechShadowInst
{
	pass pass1
	{
		CullMode = CCW;
		FillMode = Solid;
		ZEnable = true;
		ZWriteEnable = true;
		AlphaBlendEnable = false;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		PointSpriteEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader = compile vs_2_0 vsShadowInst();
		PixelShader = compile ps_2_0 psShadow();
	}
}

technique TechDrawInst
{
	pass pass1
	{
		AlphaBlendEnable = false;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		AlphaTestEnable = true;
		AlphaRef = 0xA8;
		AlphaFunc = GreaterEqual;
		PointSpriteEnable = false;
		CullMode = CCW;
		FillMode = Solid;
		StencilEnable = false;
		ZEnable = true;
		ZWriteEnable = true;
		MultiSampleAntiAlias = false;
		VertexShader = compile vs_2_0 vsDrawInst();
		PixelShader = compile ps_2_0 psDraw();
	}
}

technique TechDeferLight1
{
	pass pass1
	{
		DEFER_LIGHT_PASS_OPTIONS
		VertexShader = compile vs_2_0 vsDeferLight1();
		PixelShader = compile ps_2_0 psDeferLight();
	}
}
