#include "Inc/WarFog.fx"
#include "Inc/Shadows.fx"

float4x4	matViewProj;		//  view*proj
float4		vLightDir;			// xyz -  , w - depthbias
float3		cLightDiffuse;		//   
float3		cLightAmbient;		//  
texture		tBaseTexture;			//  
texture		tBumpTexture;
float3		vEyePos;

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sBumpSampler=sampler_state
{
	Texture=(tBumpTexture);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

struct vsInput
{
	float4 vPos		:POSITION;
	float3 vNormal	:NORMAL;
	float2 vTex		:TEXCOORD0;
	float3 vTangent	:TEXCOORD1;	// Decal Space XAxis
};

struct vsOutputBump
{
	float4 vPos			:POSITION;		// 
	float2 vTex			:TEXCOORD0;		//   
	float3 vTangent		:TEXCOORD1;		// tangent
	float3 vNormal		:TEXCOORD2;		// normal
	float3 vBinormal	:TEXCOORD3;		// binormal
	float3 vFogPos		:TEXCOORD4;
	float4 vShadowParams:TEXCOORD5;		//    
	float3 vParallax	:TEXCOORD6;
};

struct vsOutputSimple
{
	float4 vPos			:POSITION;		// 
	float2 vTex			:TEXCOORD0;		//   
	float3 vNormal		:TEXCOORD1;		// normal
	float3 vFogPos		:TEXCOORD2;
	float4 vShadowParams:TEXCOORD3;		//    
};

vsOutputBump vsDrawBump(vsInput vIn)
{
	vsOutputBump vOut;
	vOut.vTex = vIn.vTex;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vFogPos = GetWarFogCoords(vIn.vPos);
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	vOut.vNormal = vIn.vNormal;
	vOut.vTangent = vIn.vTangent;
	vOut.vBinormal = cross(vOut.vNormal, vOut.vTangent);
	float3 parallax = normalize(vEyePos - vIn.vPos);
	vOut.vParallax = float3(
		dot(parallax, vOut.vTangent), 
		dot(parallax, vOut.vNormal),
		dot(parallax, vOut.vBinormal)
		);
	return vOut;
}

vsOutputSimple vsDrawSimple(vsInput vIn)
{
	vsOutputSimple vOut;
	float4 vTPos = vIn.vPos;
	vOut.vTex = vIn.vTex;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vFogPos = GetWarFogCoords(vIn.vPos);
	vOut.vNormal = vIn.vNormal;
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	return vOut;
}


float4 psDrawBump(vsOutputBump vIn):COLOR0
{
//	parallax bump mapping
	float height = tex2D(sBumpSampler, vIn.vTex).w;
	float3 parallax = normalize(vIn.vParallax);
	float2 texcoord = vIn.vTex + parallax.xz * ((height * 2 - 1) * 0.02);
	float4 vColor = tex2D(sBaseSampler, texcoord);
	float shdValue = Draw_GetShadowSimple(vIn.vShadowParams, vLightDir.w);
	float3 vBump = tex2D(sBumpSampler, texcoord).xyz * 2 - 1;
	float3 vNormal = normalize(vBump.x * vIn.vTangent + vBump.y * vIn.vBinormal + vBump.z * vIn.vNormal);
	float3 vDiffuse = cLightDiffuse * max(dot(vNormal, -vLightDir.xyz), -0.3);
	float3 resColor = vColor.xyz * (cLightAmbient + vDiffuse) * shdValue * GetWarFog(vIn.vFogPos);
	return float4(resColor, vColor.w);
}

float4 psDrawSimple(vsOutputSimple vIn):COLOR0
{
	float4 vColor = tex2D(sBaseSampler, vIn.vTex);
	float shdValue = Draw_GetShadowSimple(vIn.vShadowParams, vLightDir.w);
	float3 vDiffuse = cLightDiffuse * max(dot(vIn.vNormal, -vLightDir.xyz), -0.3);
	float3 resColor = vColor.xyz * (cLightAmbient + vDiffuse) * shdValue * GetWarFog(vIn.vFogPos);
	return float4(resColor,vColor.w);
}

technique TechDrawBump
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZWriteEnable=false;
		ZEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsDrawBump();
		PixelShader=compile ps_2_0 psDrawBump();
	}
}

technique TechDrawSimple
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZWriteEnable=false;
		ZEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsDrawSimple();
		PixelShader=compile ps_2_0 psDrawSimple();
	}
}

