///////////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////

float4x4	matViewProj;	// view*projection
float4		vEyePos;		// xyz - eyepos, w - scale coefficient
float4		vParams;		// x - mindist (Fog=1), y - 1/(max-min)
texture		tTexture:register(t3);		//  


///////////////////////////////////////////////////////////////////////////////////
// STREAM MAPPING
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float3	vPos	:POSITION;	// 
	float4	vColor	:COLOR0;	// 
	float	fPSize	:PSIZE;		// 
};


///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sPartSampler=sampler_state
{
	Texture=(tTexture);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutput
{
    float4	vPos	:POSITION;	// 
    float4	vColor	:COLOR0;	// 
    float	fPSize	:PSIZE;		//   
    float2  vTex	:TEXCOORD0;
};

vsOutput vsDefault(vsInput vIn)
{
	vsOutput vOut;
	float fDist = length(vEyePos.xyz - vIn.vPos);
	vOut.vPos=mul(float4(vIn.vPos,1),matViewProj);
	vOut.vColor=float4(vIn.vColor.xyz,(1-saturate((fDist-vParams.x)*vParams.y)));
	vOut.fPSize=vIn.fPSize*vEyePos.w / fDist;
	vOut.vTex=0;
	return vOut;
}

float4 psDefaultHLSL(vsOutput vIn):COLOR0
{
	float4 cColor=tex2D(sPartSampler,vIn.vTex);
	float alpha=cColor.w-0.66;
	cColor=float4(cColor.xyz*cColor.w,1);
	clip(alpha);
	return cColor*vIn.vColor;
}

PixelShader psDefault = asm
{
	ps_2_0
	dcl_2d s3
	dcl v0.rgba
	dcl t0.xy
	// simulate alpha test
	texld r0,t0,s3
	sub r2.w,r0.w,c0.w
	
	mul r1,r0,v0
	mov oC0,r1
};

technique TechDefault
{
	pass pass1
	{
		AlphaBlendEnable=true;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=true;
		Sampler[3]=(sPartSampler);
		ZWriteEnable=true;
		ZEnable=true;
		CullMode=CCW;
		FillMode=Solid;
		VertexShader=compile vs_2_0 vsDefault();
		PixelShader=compile ps_2_0 psDefaultHLSL();
//		PixelShader=(psDefault);
	}
}