texture tScene;
float4x4 mInvViewProj;

sampler sScene = sampler_state
{
	Texture = (tScene);
	AddressU = Clamp;
	AddressV = Clamp;
	MinFilter = Linear;
	MipFilter = None;
	MagFilter = Linear;
};

struct vsApplyLightInput 
{
	float3 vPos:POSITION;
	float4 vLPos:TEXCOORD0;
	float4 vLColor:TEXCOORD1;
};

struct vsApplyLightOutput
{
	float4 vPos:POSITION;
	float4 vLPos:TEXCOORD0;
	float4 vLColor:TEXCOORD1;
	float2 vTPos:TEXCOORD2;
	float2 vTex:TEXCOORD3;
};

vsApplyLightOutput vsDefault(vsApplyLightInput vIn)
{
	vsApplyLightOutput vOut;
	float ofs = 0.5/256;
	vOut.vPos = float4(vIn.vPos.xy, 0, 1);
	vOut.vTPos = vIn.vPos;
	vOut.vTex = float2(vIn.vPos.x * 0.5 + 0.5 + ofs, 0.5 + ofs - vIn.vPos.y * 0.5);
	vOut.vLPos = vIn.vLPos;
	vOut.vLColor = vIn.vLColor;
	return vOut;
}

float4 psDefault(vsApplyLightOutput vIn):COLOR0
{
	float4 data = tex2D(sScene, vIn.vTex);
	float4 pos = float4(vIn.vTPos.x, vIn.vTPos.y, 1 - data.w/888, 1);
	pos = mul(pos, mInvViewProj);
	pos /= pos.w;
	float3 ld = vIn.vLPos.xyz - pos.xyz;
	float d = length(ld);
	float coeff = max(dot(data.xyz, ld / d), 0);
	float att = max(1 - d * d / vIn.vLPos.w, 0);
	return float4(vIn.vLColor.xyz * vIn.vLColor.w * att * coeff, 1);
}

technique techLightDefault
{
	pass pass1
	{
		CullMode = None;
		AlphaBlendEnable=true;
		SrcBlend = One;
		DestBlend = One;
		FillMode = Solid;
		AlphaTestEnable = true;
		AlphaRef=0x00;
		AlphaFunc=Greater;
		PointSpriteEnable=false;
		ZEnable=false;
		ZWriteEnable=false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsDefault();
		PixelShader=compile ps_2_0 psDefault();
	}
}



