// labels.fx
// Effect for drawing object labels as billboards
// Parameters

int ColorToIdx(float aValue)
{
	return D3DCOLORtoUBYTE4(aValue);
}

int2 ColorToIdx2(float2 aValue)
{
	return int2(ColorToIdx(aValue.x), ColorToIdx(aValue.y));
}


///////////////////////////////////////////////////////////////////////////////////
// PARAMETERS
///////////////////////////////////////////////////////////////////////////////////

float4x4	mViewProj;			// view*proj
float3		vBillBrdUp;			// up vector * height
float3		vBillBrdRight;		// right vector
float4		vEyePos;			// xyz - eye pos, w - scale coeff

texture		tTexture:register(t3);		//  

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sBaseSampler=sampler_state
{
	Texture=(tTexture);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};


///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

static const float4 c_ConfirmSize = { 0.125, 0.125, 0, 16 }; // in pixels

static const float4 c_Offset[] =	// xy - vertex offset, zw - texture coords
{
	{-1, -1, 0.0, 1.0, },	// bottom left
	{-1,  1, 0.0, 0.0, },	// upper left
	{ 1,  1, 1.0, 0.0, },	// upper right
	{ 1, -1, 1.0, 1.0, },	// bottom right
};

struct vsInput
{
	float3	vPos		:POSITION;		// xyz - 
//	float2  vOffset		:TEXCOORD0;
	float4	vParams		:COLOR0;		// x - frame, y - index, z - offset
};

///////////////////////////////////////////////////////////////////////////////////
// RENDER WITH SHADOW SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutput
{
	float4	vPos		:POSITION;		// 
	float2	vTex		:TEXCOORD0;		//  
};

vsOutput vsDefault(vsInput vIn)
{
	vsOutput vOut;
	float3 vTPos = vIn.vPos;
	// compute transformed position
	float size = c_ConfirmSize.w * length(vEyePos.xyz - vTPos) / vEyePos.w;
	vTPos.y += size;
	float4 coords = c_Offset[ColorToIdx(vIn.vParams.b)];
	vTPos += (vBillBrdUp * coords.y + vBillBrdRight * coords.x) * size;
	vOut.vPos = mul(float4(vTPos, 1), mViewProj);
	vOut.vTex = c_ConfirmSize.xy * ColorToIdx2(vIn.vParams.rg) + c_ConfirmSize.xy * coords.zw;
	return vOut;
}

float4 psDefault(vsOutput vIn):COLOR0
{
	float4 vColor=tex2D(sBaseSampler, vIn.vTex);
	return vColor;
}

technique TechDefault
{
	pass pass1
	{
		CullMode=None;
		FillMode=Solid;
		AlphaBlendEnable=true;
		AlphaTestEnable=false;
		AlphaRef=0xA8;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=false;
		ZWriteEnable=false;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		VertexShader=compile vs_2_0 vsDefault();
		PixelShader=compile ps_2_0 psDefault();
	}
}
