// material.fx
// Effect for drawing meshes with materials
// Parameters


///////////////////////////////////////////////////////////////////////////////////
// PARAMETERS
///////////////////////////////////////////////////////////////////////////////////

float4x4	matWorld;			// 
float4x4	matViewProj;		//  view*proj
shared float3 vLightDir;		// xyz -  
float3		vEyePos;			//  
shared float4 cLightDiffuse;	//   
shared float4 cLightAmbient;	//  
float4		cMatAmbient;		//  
float4		cMatDiffuse;		//   
float4		cMatSpecular;		//   
float		nFillMode;			//  
float4		cModulationColor;	//  
texture		tBaseTexture:register(t0);		//  


///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float3	vPos		:POSITION;		// xyz - 
	float3	vNormal		:NORMAL;		// 
	float2	vTex		:TEXCOORD0;		//  
};

///////////////////////////////////////////////////////////////////////////////////
// RENDER WITH SHADOW SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutput
{
	float4	vPos		:POSITION;		// 
	float2	vTex		:TEXCOORD0;		//   
	float4	vDiffuse	:COLOR0;		//  
	float4	vSpecular	:COLOR1;		//  
};

vsOutput vsDraw(vsInput vIn)
{
	vsOutput vOut;
	vOut.vTex=vIn.vTex;
	float3 vTPos=mul(float4(vIn.vPos,1),matWorld);
	float3 vTNormal=normalize(mul(vIn.vNormal,(float3x3)matWorld));
	float fDiffuseCoeff=clamp(dot(vTNormal,-vLightDir),-0.2,1);
	float3 vHalf=normalize(normalize(vEyePos-vTPos)-vLightDir.xyz);
	float fSpecularCoeff=dot(vTNormal,vHalf)*(fDiffuseCoeff<0 ? 0 : 1);
	fSpecularCoeff*=fSpecularCoeff;											// ^2
	fSpecularCoeff*=fSpecularCoeff;											// ^4
	fSpecularCoeff*=fSpecularCoeff;											// ^8
	fSpecularCoeff*=fSpecularCoeff;											// ^16
	vOut.vDiffuse=cMatDiffuse*cLightDiffuse*fDiffuseCoeff;					//    
	vOut.vSpecular=cMatSpecular*fSpecularCoeff;								//    
	vOut.vPos=mul(float4(vIn.vPos,1),mul(matWorld,matViewProj));
	return vOut;
}

float4 psDraw(vsOutput vIn):COLOR0
{
	float4 vColor=tex2D(sBaseSampler,vIn.vTex);
	float4 resColor=vColor*(cMatAmbient*cLightAmbient+vIn.vDiffuse)*cModulationColor;
	return float4(saturate(resColor+vIn.vSpecular).xyz,vColor.w * cModulationColor.w);
}

technique TechDraw
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=(nFillMode);
		AlphaBlendEnable=true;
		AlphaTestEnable=false;
		AlphaRef=0x40;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZWriteEnable=true;
		ZEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		StencilEnable = false;
		VertexShader=compile vs_2_0 vsDraw();
		PixelShader=compile ps_2_0 psDraw();
	}
}