//modelskin.fx
// Effect for drawing skinned multipart model
// Parameters
#include "Inc/warfog.fx"


///////////////////////////////////////////////////////////////////////////////////
// PARAMETERS
///////////////////////////////////////////////////////////////////////////////////

float4x4	matWorld;			// 
float4x4	matViewProj;		//  view*proj
float4x4	matLightViewProj;	//  view*proj
float4x4	matShadowTexProj;	//  
float4		vLightDir;			// xyz -  , w - Alpha ref fog shadow generation techniques
float4		cLightDiffuse;		//   
float4		vEyePos;			// xyz -  /Billboard up vector, w - DepthBias
float4		cAmbient;			//   (*)
float4		cMatDiffuse;		//   
float4		cMatSpecular;		//   
float4		vParams;			// x - FrameOffset, y - FillMode
float4x4	arrBones[36];		//   
float4		arrFrames[80];		//  
								// x - BoneId, y - FrameDamage, z - FrameVisible
float3		vBillBrdRight;		//     
float4		cModelColor;		//   
texture		tBaseTexture:register(t3);		//  
texture		tDamageTexture:register(t1);		// 3  
texture		tBumpTexture:register(t2);		//  
texture		tShadowMap:register(t7);			//  


///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sDamageSampler=sampler_state
{
	Texture=(tDamageTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sBumpSampler=sampler_state
{
	Texture=(tBumpTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sShadowSampler=sampler_state
{
	Texture=(tShadowMap);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float3	vNormal		:NORMAL;		// 
	float2	vTex		:TEXCOORD0;		//  
};

struct vsInputP
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float	fPSize		:PSIZE;			//  
	float3	vNormal		:NORMAL;		//  
	float4	vColor		:COLOR0;		//  
	float2	vTex		:TEXCOORD0;		//  
	float2	vOffset		:TEXCOORD1;		//    
};

///////////////////////////////////////////////////////////////////////////////////
// SHADOW GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputShadowGen
{
	float4	vPos		:POSITION;		// 	
	float2	vTex		:TEXCOORD0;		//  
	float	fDepth		:TEXCOORD1;		//   
};

struct vsOutputShadowGenP
{
	float4	vPos		:POSITION;		// 
	float4	vColor		:COLOR0;		//  
	float2	vTex		:TEXCOORD0;		//  
	float	fDepth		:TEXCOORD1;		//   
};

vsOutputShadowGen vsShadowGen(vsInput vIn)
{
	vsOutputShadowGen vOut;
	//   
	float4 vFrameParams=arrFrames[vIn.vPos.w-vParams.x];
	//   
	vOut.vTex=vIn.vTex;
	//    
	float4x4 mFWorld=mul(arrBones[vFrameParams.x],matWorld);
	// compute transformed position
	float3 vTPos=mul(float4(vIn.vPos.xyz,1),mFWorld);
	// compute resulting position
	vOut.vPos=mul(float4(vTPos,1),matLightViewProj)*vFrameParams.z;
	vOut.fDepth=vOut.vPos.z;
	return vOut;	
}

vsOutputShadowGenP vsShadowGenP(vsInputP vIn)
{
	vsOutputShadowGenP vOut;
	float4 vFrameParams=arrFrames[vIn.vPos.w-vParams.x];
	float4x4 mFWorld=mul(arrBones[vFrameParams.x],matWorld);
	// compute transformed position
	float3 vTPos=mul(float4(vIn.vPos.xyz,1),mFWorld);
	// for particles vEyePos.xyz=vBillBrdUp
	vTPos+=(vEyePos.xyz*vIn.vOffset.x+vBillBrdRight*vIn.vOffset.y)*vIn.fPSize;
	// compute resulting position
	vOut.vPos=mul(float4(vTPos,1),matLightViewProj)*vFrameParams.z;
	vOut.vColor=vIn.vColor;
	vOut.fDepth=vOut.vPos.z;
	vOut.vTex=vIn.vTex;
	return vOut;
}

float4 psShadowGen(vsOutputShadowGen vIn): COLOR0
{
	float4 c = tex2D(sBaseSampler, vIn.vTex);
	clip(c.w-vLightDir.w);
	return vIn.fDepth;
}

PixelShader psShadowGenAsm=asm
{
	ps_2_0
//	dcl v0.rgba
	dcl t0.xy
	dcl t1.x
	dcl_2d s0
	def c1,1,1,1,1
	// sample model texture
	texld r0,t0,s0
//	mul r1,r0,v0
	// subtract alpha ref from loaded alpha
	sub r1,r0.w,c0.w
	// check for transparent and kill transparent pixel
	mov r2,t1.x
	cmp r0,r1,r2,c1
	mov oC0,r0
	mov oDepth,r0.x
};

float4 psShadowGenP(vsOutputShadowGenP vIn): COLOR0
{
	float4 c = tex2D(sBaseSampler, vIn.vTex);
	clip(c.w-vLightDir.w);
	return vIn.fDepth;	
}

PixelShader psShadowGenPAsm=asm
{
	ps_2_0
	dcl v0.rgba
	dcl t0.xy
	dcl t1.x
	dcl_2d s0
	def c1,1,1,1,1
	// sample model texture
	texld r0,t0,s0
//	mul r1,r0,v0
	// subtract alpha ref from loaded alpha
	sub r2,r0.w,c0.w
	// check for transparent and kill transparent pixel
	mov r3,t1.x
	cmp r0,r2,r3,c1
	mov oC0,r0
	mov oDepth,r0.x
};



technique TechShadowGen
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		Sampler[0]=(sBaseSampler);
		PixelShaderConstant[0]=(vLightDir);
		VertexShader=compile vs_2_0 vsShadowGen();
		PixelShader=compile ps_2_0 psShadowGen(); //(psShadowGenAsm);
	}
}

technique TechShadowGenP
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		Sampler[0]=(sBaseSampler);
		PixelShaderConstant[0]=(vLightDir);
		VertexShader=compile vs_2_0 vsShadowGenP();
		PixelShader=compile ps_2_0 psShadowGenP(); //(psShadowGenPAsm);
	}
}

///////////////////////////////////////////////////////////////////////////////////
// RENDER WITH SHADOW SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutput
{
	float4	vPos		:POSITION;		// 
	float2	vBaseTex	:TEXCOORD0;		//   
	float3	vDamageTex	:TEXCOORD1;		//    
	float4	vDiffuse	:COLOR0;		//  
	float4	vSpecular	:COLOR1;		//  
	float2  vShadowTex	:TEXCOORD2;		//  
	float	fDepth		:TEXCOORD3;		//   
	float2  vFogPos		:TEXCOORD4;
};

struct vsOutputP
{
	float4	vPos		:POSITION;		// 
	float2	vTex		:TEXCOORD0;		//  
	float3	vDiffuse	:COLOR0;		//  
	float2	vShadowTex	:TEXCOORD1;		//  
	float	fDepth		:TEXCOORD2;		//   
	float2  vFogPos		:TEXCOORD3;
};

vsOutput vsOutputShadow(vsInput vIn)
{
	vsOutput vOut;
	float4 vFrameParams=arrFrames[vIn.vPos.w-vParams.x];					//    
	vOut.vDamageTex=float3(vIn.vTex,vFrameParams.y);						//    
	vOut.vBaseTex=vIn.vTex;													//    
	float4x4 mFWorld=mul(arrBones[vFrameParams.x],matWorld);				//   
	float4 vTPos=float4(mul(float4(vIn.vPos.xyz,1),mFWorld).xyz,1);		//   
	float3 vTNormal=normalize(mul(vIn.vNormal,(float3x3)mFWorld));			//   
	float fDiffuseCoeff=clamp(dot(vTNormal,-vLightDir.xyz),-0.2,1);				//    
	float3 vHalf=normalize(normalize(vEyePos.xyz-vTPos.xyz)-vLightDir.xyz);		//   
	float fSpecularCoeff=dot(vTNormal,vHalf)*(fDiffuseCoeff<0 ? 0 : 1);		//    
	fSpecularCoeff*=fSpecularCoeff;											// ^2
	fSpecularCoeff*=fSpecularCoeff;											// ^4
	fSpecularCoeff*=fSpecularCoeff;											// ^8
	fSpecularCoeff*=fSpecularCoeff;											// ^16
	vOut.vDiffuse=cMatDiffuse * cLightDiffuse * fDiffuseCoeff;				//    
	vOut.vSpecular=cMatSpecular*fSpecularCoeff;								//    
	vOut.vPos=mul(vTPos,matViewProj)*vFrameParams.z;						//   
	vOut.vShadowTex=mul(vTPos,matShadowTexProj);							//   
	vOut.fDepth=mul(vTPos,matLightViewProj).z;								//    
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
	return vOut;
}

float4 psOutputShadow(vsOutput vIn):COLOR0
{
	float4 vColor=tex2D(sBaseSampler,vIn.vBaseTex);
	float fDepth=vIn.fDepth-vEyePos.w;
	float shdValue=((fDepth-tex2D(sShadowSampler,vIn.vShadowTex))>0 ? 0 : 1);
	float4 resColor=vColor*(cAmbient+(0.1+0.9*shdValue)*vIn.vDiffuse);
	resColor=saturate(resColor+shdValue*vIn.vSpecular);
	return float4((resColor*tex3D(sDamageSampler,vIn.vDamageTex)).xyz * GetWarFog(vIn.vFogPos),vColor.w)*cModelColor;
}

vsOutputP vsOutputShadowP(vsInputP vIn)
{
	vsOutputP vOut;
	float4 vFrameParams=arrFrames[vIn.vPos.w-vParams.x];
	float4x4 mFWorld=mul(arrBones[vFrameParams.x],matWorld);
	// compute transformed position
	float3 vTPos=mul(float4(vIn.vPos.xyz,1),mFWorld);
	vTPos+=(vEyePos.xyz*vIn.vOffset.x+vBillBrdRight*vIn.vOffset.y)*vIn.fPSize;
	float3 vTNormal=normalize(mul(vIn.vNormal,(float3x3)mFWorld));			//   
//	float fDiffuseCoeff=clamp(dot(vTNormal,-vLightDir.xyz),-0.2,1);				//    
	float fDiffuseCoeff=dot(vTNormal,-vLightDir.xyz);				//    
	fDiffuseCoeff = fDiffuseCoeff*fDiffuseCoeff*fDiffuseCoeff*2;
//	if (fDiffuseCoeff<0) fDiffuseCoeff = fDiffuseCoeff*2;
	// compute resulting position
	vOut.vPos=mul(float4(vTPos,1),matViewProj)*vFrameParams.z;	
    vOut.vDiffuse = cMatDiffuse.xyz * cLightDiffuse.xyz * fDiffuseCoeff;
	vOut.fDepth=mul(float4(vTPos,1),matLightViewProj).z;
	vOut.vTex=vIn.vTex;
	vOut.vShadowTex=mul(float4(vTPos,1),matShadowTexProj);
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
    return vOut;    	
}

float4 psOutputShadowP(vsOutputP vIn):COLOR0
{
	float4 vColor=tex2D(sBaseSampler,vIn.vTex);
//	float fDepth=vIn.fDepth-vEyePos.w;
//	float shdValue=((fDepth-tex2D(sShadowSampler,vIn.vShadowTex))>0 ? 0 : 1);
//	return vColor*float4((cAmbient+(0.1+0.9*shdValue)*vIn.vDiffuse.xyz),1)*cModelColor;
	float3 color = vColor.xyz * (cAmbient.xyz + vIn.vDiffuse.xyz) * GetWarFog(vIn.vFogPos);
	return float4(color, vColor.w) * cModelColor;
}

technique TechOutputShadow
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=(vParams.y);
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaTestEnable=true;
		AlphaRef=0x40;
		AlphaFunc=GreaterEqual;
		VertexShader=compile vs_2_0 vsOutputShadow();
		PixelShader=compile ps_2_0 psOutputShadow();
	}
}

technique TechOutputShadowP
{
	pass pass1
	{
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaTestEnable=true;
		AlphaRef=0xA8;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=true;
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		VertexShader=compile vs_2_0 vsOutputShadowP();
		PixelShader=compile ps_2_0 psOutputShadowP();
	}
}
