// mskin.fx
// Effect for drawing skinned multipart model
// Parameters

#define USE_INDEX_MAP

#include "Inc/utils.fx"
#include "Inc/warfog.fx"
#include "Inc/Shadows.fx"
#include "Inc/PointLight.fx"

#define iAlphaRefValue 0x40
#define fAlphaRefValue 0.25

///////////////////////////////////////////////////////////////////////////////////
// PARAMETERS
///////////////////////////////////////////////////////////////////////////////////

// camera
float4x4	matWorld;			// 
float4x4	matViewProj;		//  view*proj
// light
float4		vLightDir;			// xyz -  , w - depth bias
float3		cLightDiffuse;		//   
float3		cLightAmbient;		// ambient
// material
float3		cMatAmbient;			//   (*)
float3		cMatDiffuse;		//   
texture		tBaseTexture;		//  
texture		tDamageTexture;		// 3  
// model
float4		vParams;			// x - FrameOffset, y - FillMode
float4x4	arrBones[36];		//   
float4		arrFrames[80];		//  
								// x - BoneId, y - FrameDamage, z - FrameVisible

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sDamageSampler=sampler_state
{
	Texture=(tDamageTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float3	vNormal		:NORMAL;		// 
	float2	vTex		:TEXCOORD0;		//  
};

///////////////////////////////////////////////////////////////////////////////////
// SHADOW GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputShadow
{
	float4	vPos		:POSITION;		// 	
	float2	vTex		:TEXCOORD0;		//  
#ifdef FLOATSHADOWS	
	float	fDepth		:TEXCOORD1;		//   
#endif
};

float IsVisible(float aDamage)
{
	return aDamage >= 0 ? 1 : 0;
}

void ShadowGen(out vsOutputShadow vOut, vsInput vIn, float4x4 aTransform, float aVisible)
{
	vOut.vTex = vIn.vTex;
	// compute transformed position
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), aTransform);
	// compute resulting position
	vOut.vPos = mul(vTPos, matLightViewProj) * aVisible;
#ifdef FLOATSHADOWS	
	vOut.fDepth = vOut.vPos.z;
#endif
}

vsOutputShadow vsShadow(vsInput vIn)
{
	vsOutputShadow vOut;
	//   
	int aFrame = ToIndex(vIn.vPos.w - vParams.x);
	float4 vFrameParams = arrFrames[aFrame];
	//    
	vOut.vTex = vIn.vTex;
	// compute transformed position
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);
	vTPos = mul(vTPos, matWorld);
	// compute resulting position
	vOut.vPos = mul(vTPos, matLightViewProj) * vFrameParams.z;
#ifdef FLOATSHADOWS	
	vOut.fDepth = vOut.vPos.z;
#endif
	return vOut;	
}

float4 psShadow(vsOutputShadow vIn): COLOR0
{
	clip(tex2D(sBaseSampler, vIn.vTex).w - fAlphaRefValue);
#ifdef FLOATSHADOWS
	return vIn.fDepth;
#else
	return float4(0,0,0,1);
#endif
}

technique TechShadow
{
	pass pass1
	{
		CullMode = CCW;
		FillMode = Solid;
		ZEnable = true;
		ZWriteEnable = true;
		AlphaBlendEnable = false;
		AlphaTestEnable = false;
		AlphaFunc = GreaterEqual;
		PointSpriteEnable = false;
		StencilEnable = false;
		VertexShader = compile vs_2_0 vsShadow();
		PixelShader = compile ps_2_0 psShadow();
	}
}
///////////////////////////////////////////////////////////////////////////////////
// RENDER FOR DEFERRED LIGHTING
///////////////////////////////////////////////////////////////////////////////////

struct vsModelDeferLightOutput
{
	float4 vPos	: POSITION;
	float4 vTPos : TEXCOORD0;
	float3 vTNormal: TEXCOORD1;
	float2 vTex: TEXCOORD2;
};

void DrawDeferLight(out vsModelDeferLightOutput vOut, vsInput vIn, float4x4 aTransform, float aVisible)
{
	vOut.vTex = vIn.vTex;
	vOut.vTPos = mul(mul(float4(vIn.vPos.xyz, 1), aTransform), matViewProj) * aVisible;
	vOut.vPos = vOut.vTPos;
	vOut.vTNormal = mul(vIn.vNormal, (float3x3)aTransform);
}

vsModelDeferLightOutput vsDeferLight(vsInput vIn)
{
	vsModelDeferLightOutput vOut;
	vOut.vTex = vIn.vTex;
	int aFrame = ToIndex(vIn.vPos.w - vParams.x);
	float4 vFrameParams = arrFrames[aFrame];					//    
	vOut.vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);		//   
	vOut.vTPos = mul(mul(vOut.vTPos, matWorld), matViewProj)*vFrameParams.z;
	vOut.vTNormal = mul(vIn.vNormal, (float3x3)arrBones[ToIndex(vFrameParams.x)]);			//   
	vOut.vTNormal = mul(vOut.vTNormal, (float3x3)matWorld);
	vOut.vPos = vOut.vTPos;
	return vOut;
}

float4 psDeferLight(vsModelDeferLightOutput vIn): COLOR0
{
	clip(tex2D(sBaseSampler, vIn.vTex).w - fAlphaRefValue);
	return CalcDeferLightPSColor(vIn.vTPos, vIn.vTNormal);	
}

TECH_DEFER_LIGHT_BEGIN
	VertexShader = compile vs_2_0 vsDeferLight();
	PixelShader =compile ps_2_0 psDeferLight();
TECH_DEFER_LIGHT_END

///////////////////////////////////////////////////////////////////////////////////
// RENDER WITH SHADOW SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputDraw
{
	float4 vPos			:POSITION;		// 
	float3 vDiffuse		:COLOR0;		//  
	float3 vAmbient		:COLOR1;
	float3 vTex			:TEXCOORD0;		//   
	float4 vShadowParams:TEXCOORD1;		//  
	float3 vFogPos		:TEXCOORD2;
};

void Draw(out vsOutputDraw vOut, vsInput vIn, float4x4 aTransform, float aVisible, float aDamage)
{
	vOut.vTex = float3(vIn.vTex, aDamage);						//    
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), aTransform);		//   
	float3 vTNormal = mul(vIn.vNormal, (float3x3)aTransform);			//   
#ifdef NORMALIZE_NORMAL		
	vTNormal = normalize(vTNormal);
#endif
	float fDiffuseCoeff = max(dot(vTNormal, -vLightDir.xyz), -0.3);				//    
	vOut.vDiffuse = cMatDiffuse * cLightDiffuse * fDiffuseCoeff;				//    
	vOut.vAmbient = cMatAmbient * cLightAmbient;
	vOut.vPos = mul(vTPos, matViewProj) * aVisible;						//   
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
}

vsOutputDraw vsDraw(vsInput vIn)
{
	vsOutputDraw vOut;
	int aFrame = ToIndex(vIn.vPos.w - vParams.x);
	float4 vFrameParams = arrFrames[aFrame];					//    
	vOut.vTex = float3(vIn.vTex, vFrameParams.y);						//    
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);		//   
	vTPos = mul(vTPos, matWorld);
	float3 vTNormal = mul(vIn.vNormal, (float3x3)arrBones[ToIndex(vFrameParams.x)]);			//   
	vTNormal = mul(vTNormal, (float3x3)matWorld);
#ifdef NORMALIZE_NORMAL
	vTNormal = normalize(vTNormal);
#endif
	float fDiffuseCoeff = max(dot(vTNormal, -vLightDir.xyz), -0.3);				//    
	vOut.vDiffuse = cMatDiffuse * cLightDiffuse * fDiffuseCoeff;				//    
	vOut.vAmbient = cMatAmbient * cLightAmbient;
	vOut.vPos = mul(vTPos, matViewProj) * vFrameParams.z;						//   
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
	return vOut;
}

float4 psDraw(vsOutputDraw vIn):COLOR0
{
	float4 vColor = tex2D(sBaseSampler, vIn.vTex.xy);
	float shdValue = Draw_GetShadowValue(vIn.vShadowParams, vLightDir.w);
	float3 resColor = vColor.xyz * (vIn.vAmbient + (0.1 + 0.9 * shdValue) * vIn.vDiffuse);
	return float4(resColor * tex3D(sDamageSampler, vIn.vTex).xyz * GetWarFog(vIn.vFogPos), vColor.w);
}

technique TechDraw
{
	pass pass1
	{
		CullMode = CCW;
		FillMode = Solid;
		AlphaBlendEnable = false;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		PointSpriteEnable = false;
		ZEnable = true;
		ZWriteEnable = true;
		AlphaTestEnable = true;
		AlphaRef = iAlphaRefValue;
		AlphaFunc = GreaterEqual;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader = compile vs_2_0 vsDraw();
		PixelShader = compile ps_2_0 psDraw();
	}
}

/////////////////////////////////////////////////////////////////////////////////
// Instancing 1 frame
/////////////////////////////////////////////////////////////////////////////////

struct vsInputInstance1
{
	float4 vPos			:POSITION;		// Position
	float3 vNormal		:NORMAL;		// Normal
	float2 vTex			:TEXCOORD0;		// Texcoord
	float4 vInstLine1	:TEXCOORD1;		// TransposeWorldMatrix line1
	float4 vInstLine2	:TEXCOORD2;		// TransposeWorldMatrix line2
	float4 vInstLine3	:TEXCOORD3;		// TransposeWorldMatrix line3
	float  vInstLine4	:TEXCOORD4;		// y - frame damage
};

vsOutputShadow vsShadowInst1(vsInputInstance1 vIn)
{
	vsOutputShadow vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	ShadowGen(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(vIn.vInstLine4.x));
	return vOut;
};

vsOutputDraw vsDrawInst1(vsInputInstance1 vIn)
{
	vsOutputDraw vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	Draw(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(vIn.vInstLine4.x), vIn.vInstLine4.x);
	return vOut;
}

vsModelDeferLightOutput vsDeferLight1(vsInputInstance1 vIn)
{
	vsModelDeferLightOutput vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	DrawDeferLight(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(vIn.vInstLine4.x));
	return vOut;
}

technique TechShadowInst1
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable = false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsShadowInst1();
		PixelShader=compile ps_2_0 psShadow();
	}
}

technique TechDrawInst1
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaTestEnable=true;
		AlphaRef=iAlphaRefValue;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader=compile vs_2_0 vsDrawInst1();
		PixelShader=compile ps_2_0 psDraw();
	}
}

technique TechDeferLight1
{
	pass pass1
	{
		DEFER_LIGHT_PASS_OPTIONS
		VertexShader = compile vs_2_0 vsDeferLight1();
		PixelShader = compile ps_2_0 psDeferLight();
	}
}


/////////////////////////////////////////////////////////////////////////////////
// Instancing 4 frames
/////////////////////////////////////////////////////////////////////////////////

struct vsInputInstance4
{
	float4 vPos			:POSITION;		// Position
	float3 vNormal		:NORMAL;		// Normal
	float2 vTex			:TEXCOORD0;		// Texcoord
	float4 vInstLine1	:TEXCOORD1;		// TransposeWorldMatrix line1
	float4 vInstLine2	:TEXCOORD2;		// TransposeWorldMatrix line2
	float4 vInstLine3	:TEXCOORD3;		// TransposeWorldMatrix line3
	float4 vInstLine4	:TEXCOORD4;		// frame damages
	float  vInstLine5	:TEXCOORD5;		// start frame
};

vsOutputShadow vsShadowInst4(vsInputInstance4 vIn)
{
	vsOutputShadow vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine5.x);
	float dmg = vIn.vInstLine4[aFrame];
	ShadowGen(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}

vsOutputDraw vsDrawInst4(vsInputInstance4 vIn)
{
	vsOutputDraw vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine5.x);
	float dmg = vIn.vInstLine4[aFrame];
	Draw(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg), dmg);
	return vOut;	
}

vsModelDeferLightOutput vsDeferLight4(vsInputInstance4 vIn)
{
	vsModelDeferLightOutput vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine5.x);
	float dmg = vIn.vInstLine4[aFrame];
	DrawDeferLight(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}


technique TechShadowInst4
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable = false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsShadowInst4();
		PixelShader=compile ps_2_0 psShadow();
	}
}

technique TechDrawInst4
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaTestEnable=true;
		AlphaRef=iAlphaRefValue;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader=compile vs_2_0 vsDrawInst4();
		PixelShader=compile ps_2_0 psDraw();
	}
}

technique TechDeferLight4
{
	pass pass1
	{
		DEFER_LIGHT_PASS_OPTIONS
		VertexShader = compile vs_2_0 vsDeferLight4();
		PixelShader = compile ps_2_0 psDeferLight();
	}
}


/////////////////////////////////////////////////////////////////////////////////
// Instancing 16 frames
/////////////////////////////////////////////////////////////////////////////////

struct vsInputInstance16
{
	float4 vPos			:POSITION;		// Position
	float3 vNormal		:NORMAL;		// Normal
	float2 vTex			:TEXCOORD0;		// Texcoord
	float4 vInstLine1	:TEXCOORD1;		// TransposeWorldMatrix line1
	float4 vInstLine2	:TEXCOORD2;		// TransposeWorldMatrix line2
	float4 vInstLine3	:TEXCOORD3;		// TransposeWorldMatrix line3
	float4 vInstLine4	:TEXCOORD4;		// frame damages 0 - 3
	float4 vInstLine5	:TEXCOORD5;		// frame damages 4 - 7
	float4 vInstLine6	:TEXCOORD6;		// frame damages 8 - 11
	float4 vInstLine7	:TEXCOORD7;		// frame damages 12 - 15
	float  vInstLine8	:TEXCOORD8;		// start frame
};

static const int2 c_Indices16[16] = 
{
	0, 0,
	0, 1,
	0, 2,
	0, 3,
	1, 0,
	1, 1,
	1, 2,
	1, 3,
	2, 0,
	2, 1,
	2, 2,
	2, 3,
	3, 0,
	3, 1,
	3, 2,
	3, 3,
};

vsOutputShadow vsShadowInst16(vsInputInstance16 vIn)
{
	vsOutputShadow vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine8.x);
	float4 damages[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7};
#ifdef USE_INDEX_MAP
	int2 idx = c_Indices16[aFrame];
	float dmg = damages[idx.x][idx.y];
#else
	int i1 = ToIndex(aFrame / 4);
	int i2 = ToIndex(frac(aFrame / 4) * 4);
	float dmg = damages[i1][i2];
#endif	
	ShadowGen(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}

vsOutputDraw vsDrawInst16(vsInputInstance16 vIn)
{
	vsOutputDraw vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine8.x);
	float4 damages[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7};
#ifdef USE_INDEX_MAP
	int2 idx = c_Indices16[aFrame];
	float dmg = damages[idx.x][idx.y];
#else
	int i1 = ToIndex(aFrame / 4);
	int i2 = ToIndex(frac(aFrame / 4) * 4);
	float dmg = damages[i1][i2];
#endif	
	Draw(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg), dmg);
	return vOut;	
}

vsModelDeferLightOutput vsDeferLight16(vsInputInstance16 vIn)
{
	vsModelDeferLightOutput vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine8.x);
	float4 damages[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7};
#ifdef USE_INDEX_MAP
	int2 idx = c_Indices16[aFrame];
	float dmg = damages[idx.x][idx.y];
#else
	int i1 = ToIndex(aFrame / 4);
	int i2 = ToIndex(frac(aFrame / 4) * 4);
	float dmg = damages[i1][i2];
#endif	
	DrawDeferLight(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}

technique TechShadowInst16
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable = false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsShadowInst16();
		PixelShader=compile ps_2_0 psShadow();
	}
}

technique TechDrawInst16
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaTestEnable=true;
		AlphaRef=iAlphaRefValue;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader=compile vs_2_0 vsDrawInst16();
		PixelShader=compile ps_2_0 psDraw();
	}
}

technique TechDeferLight16
{
	pass pass1
	{
		DEFER_LIGHT_PASS_OPTIONS
		VertexShader = compile vs_2_0 vsDeferLight16();
		PixelShader = compile ps_2_0 psDeferLight();
	}
}


/////////////////////////////////////////////////////////////////////////////////
// Instancing 24 frames
/////////////////////////////////////////////////////////////////////////////////

struct vsInputInstance24
{
	float4 vPos			:POSITION;		// Position
	float3 vNormal		:NORMAL;		// Normal
	float2 vTex			:TEXCOORD0;		// Texcoord
	float4 vInstLine1	:TEXCOORD1;		// TransposeWorldMatrix line1
	float4 vInstLine2	:TEXCOORD2;		// TransposeWorldMatrix line2
	float4 vInstLine3	:TEXCOORD3;		// TransposeWorldMatrix line3
	float4 vInstLine4	:TEXCOORD4;		// frame damages 0 - 3
	float4 vInstLine5	:TEXCOORD5;		// frame damages 4 - 7
	float4 vInstLine6	:TEXCOORD6;		// frame damages 8 - 11
	float4 vInstLine7	:TEXCOORD7;		// frame damages 12 - 15
	float4 vInstLine8	:TEXCOORD8;		// frame damages 16 - 19
	float4 vInstLine9	:TEXCOORD9;		// frame damages 20 - 23
	float  vInstLine10	:TEXCOORD10;	// start frame
};

static const int3 c_Indices24[24] = 
{
	0, 0, 0,
	0, 0, 1,
	0, 0, 2,
	0, 0, 3,
	0, 1, 0,
	0, 1, 1,
	0, 1, 2,
	0, 1, 3,
	0, 2, 0,
	0, 2, 1,
	0, 2, 2,
	0, 2, 3,
	0, 3, 0,
	0, 3, 1,
	0, 3, 2,
	0, 3, 3,
	1, 0, 0,
	1, 0, 1,
	1, 0, 2,
	1, 0, 3,
	1, 1, 0,
	1, 1, 1,
	1, 1, 2,
	1, 1, 3,
};

vsOutputShadow vsShadowInst24(vsInputInstance24 vIn)
{
	vsOutputShadow vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	float4 damages1[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7 };
	float4 damages2[2] = { vIn.vInstLine8, vIn.vInstLine9 };
#ifdef USE_INDEX_MAP
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine10.x);
	int3 idx = c_Indices24[aFrame];
	float2 tdmg = float2(damages1[idx.y][idx.z], damages2[idx.y][idx.z]);
	float dmg = tdmg[idx.x];
#else
	float frm = vIn.vPos.w - vIn.vInstLine10.x;
	float t1 = frm / 16;
	float t2 = frac(t1)*16;
	int i1 = ToIndex(t1);
	int i2 = ToIndex(t2/4);
	int i3 = ToIndex(frac(frm / 4) * 4);
	float2 tdmg = float2(damages1[i2][i3], damages2[i2][i3]);
	float dmg = tdmg[i1];
#endif	
	ShadowGen(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}

vsOutputDraw vsDrawInst24(vsInputInstance24 vIn)
{
	vsOutputDraw vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	float4 damages1[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7 };
	float4 damages2[2] = { vIn.vInstLine8, vIn.vInstLine9 };
#ifdef USE_INDEX_MAP
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine10.x);
	int3 idx = c_Indices24[aFrame];
	float2 tdmg = float2(damages1[idx.y][idx.z], damages2[idx.y][idx.z]);
	float dmg = tdmg[idx.x];
#else
	float frm = vIn.vPos.w - vIn.vInstLine10.x;
	float t1 = frm / 16;
	float t2 = frac(t1)*16;
	int i1 = ToIndex(t1);
	int i2 = ToIndex(t2/4);
	int i3 = ToIndex(frac(frm / 4) * 4);
	float2 tdmg = float2(damages1[i2][i3], damages2[i2][i3]);
	float dmg = tdmg[i1];
#endif	
	Draw(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg), dmg);
	return vOut;	
}

vsModelDeferLightOutput vsDeferLight24(vsInputInstance24 vIn)
{
	vsModelDeferLightOutput vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	float4 damages1[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7 };
	float4 damages2[2] = { vIn.vInstLine8, vIn.vInstLine9 };
#ifdef USE_INDEX_MAP
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine10.x);
	int3 idx = c_Indices24[aFrame];
	float2 tdmg = float2(damages1[idx.y][idx.z], damages2[idx.y][idx.z]);
	float dmg = tdmg[idx.x];
#else
	float frm = vIn.vPos.w - vIn.vInstLine10.x;
	float t1 = frm / 16;
	float t2 = frac(t1)*16;
	int i1 = ToIndex(t1);
	int i2 = ToIndex(t2/4);
	int i3 = ToIndex(frac(frm / 4) * 4);
	float2 tdmg = float2(damages1[i2][i3], damages2[i2][i3]);
	float dmg = tdmg[i1];
#endif	
	DrawDeferLight(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}


technique TechShadowInst24
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable = false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsShadowInst24();
		PixelShader=compile ps_2_0 psShadow();
	}
}

technique TechDrawInst24
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaTestEnable=true;
		AlphaRef=iAlphaRefValue;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader=compile vs_2_0 vsDrawInst24();
		PixelShader=compile ps_2_0 psDraw();
	}
}

technique TechDeferLight24
{
	pass pass1
	{
		DEFER_LIGHT_PASS_OPTIONS
		VertexShader = compile vs_2_0 vsDeferLight24();
		PixelShader = compile ps_2_0 psDeferLight();
	}
}


/////////////////////////////////////////////////////////////////////////////////
// Instancing 32 frames
/////////////////////////////////////////////////////////////////////////////////

struct vsInputInstance32
{
	float4 vPos			:POSITION;		// Position
	float3 vNormal		:NORMAL;		// Normal
	float2 vTex			:TEXCOORD0;		// Texcoord
	float4 vInstLine1	:TEXCOORD1;		// TransposeWorldMatrix line1
	float4 vInstLine2	:TEXCOORD2;		// TransposeWorldMatrix line2
	float4 vInstLine3	:TEXCOORD3;		// TransposeWorldMatrix line3
	float4 vInstLine4	:TEXCOORD4;		// frame damages 0 - 3
	float4 vInstLine5	:TEXCOORD5;		// frame damages 4 - 7
	float4 vInstLine6	:TEXCOORD6;		// frame damages 8 - 11
	float4 vInstLine7	:TEXCOORD7;		// frame damages 12 - 15
	float4 vInstLine8	:TEXCOORD8;		// frame damages 16 - 19
	float4 vInstLine9	:TEXCOORD9;		// frame damages 20 - 23
	float4 vInstLine10	:TEXCOORD10;	// frame damages 24 - 27
	float4 vInstLine11	:TEXCOORD11;	// frame damages 28 - 31
	float  vInstLine12	:TEXCOORD12;	// start frame
};

static const int3 c_Indices32[32] = 
{
	0, 0, 0,
	0, 0, 1,
	0, 0, 2,
	0, 0, 3,
	0, 1, 0,
	0, 1, 1,
	0, 1, 2,
	0, 1, 3,
	0, 2, 0,
	0, 2, 1,
	0, 2, 2,
	0, 2, 3,
	0, 3, 0,
	0, 3, 1,
	0, 3, 2,
	0, 3, 3,
	1, 0, 0,
	1, 0, 1,
	1, 0, 2,
	1, 0, 3,
	1, 1, 0,
	1, 1, 1,
	1, 1, 2,
	1, 1, 3,
	1, 2, 0,
	1, 2, 1,
	1, 2, 2,
	1, 2, 3,
	1, 3, 0,
	1, 3, 1,
	1, 3, 2,
	1, 3, 3,
};

vsOutputShadow vsShadowInst32(vsInputInstance32 vIn)
{
	vsOutputShadow vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	float4 damages1[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7 };
	float4 damages2[4] = { vIn.vInstLine8, vIn.vInstLine9, vIn.vInstLine10, vIn.vInstLine11 };
#ifdef USE_INDEX_MAP
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine12.x);
	int3 idx = c_Indices24[aFrame];
	float2 tdmg = float2(damages1[idx.y][idx.z], damages2[idx.y][idx.z]);
	float dmg = tdmg[idx.x];
#else
	float frm = vIn.vPos.w - vIn.vInstLine12.x;
	float t1 = frm / 16;
	float t2 = frac(t1)*16;
	int i1 = ToIndex(t1);
	int i2 = ToIndex(t2/4);
	int i3 = ToIndex(frac(frm / 4) * 4);
	float2 tdmg = float2(damages1[i2][i3], damages2[i2][i3]);
	float dmg = tdmg[i1];
#endif	
	ShadowGen(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}

vsOutputDraw vsDrawInst32(vsInputInstance32 vIn)
{
	vsOutputDraw vOut;
	float4x4 mFWorld = float4x4(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	float4 damages1[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7 };
	float4 damages2[4] = { vIn.vInstLine8, vIn.vInstLine9, vIn.vInstLine10, vIn.vInstLine11 };
#ifdef USE_INDEX_MAP
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine12.x);
	int3 idx = c_Indices24[aFrame];
	float2 tdmg = float2(damages1[idx.y][idx.z], damages2[idx.y][idx.z]);
	float dmg = tdmg[idx.x];
#else
	float frm = vIn.vPos.w - vIn.vInstLine12.x;
	float t1 = frm / 16;
	float t2 = frac(t1)*16;
	int i1 = ToIndex(t1);
	int i2 = ToIndex(t2/4);
	int i3 = ToIndex(frac(frm / 4) * 4);
	float2 tdmg = float2(damages1[i2][i3], damages2[i2][i3]);
	float dmg = tdmg[i1];
#endif	
	Draw(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg), dmg);
	return vOut;	
}

vsModelDeferLightOutput vsDeferLight32(vsInputInstance32 vIn)
{
	vsModelDeferLightOutput vOut;
	float4x4 mFWorld = float4x4
	(
		vIn.vInstLine1, 
		vIn.vInstLine2,
		vIn.vInstLine3,
		0, 0, 0, 1
	);
	float4 damages1[4] = { vIn.vInstLine4, vIn.vInstLine5, vIn.vInstLine6, vIn.vInstLine7 };
	float4 damages2[4] = { vIn.vInstLine8, vIn.vInstLine9, vIn.vInstLine10, vIn.vInstLine11 };
#ifdef USE_INDEX_MAP
	int aFrame = ToIndex(vIn.vPos.w - vIn.vInstLine12.x);
	int3 idx = c_Indices24[aFrame];
	float2 tdmg = float2(damages1[idx.y][idx.z], damages2[idx.y][idx.z]);
	float dmg = tdmg[idx.x];
#else
	float frm = vIn.vPos.w - vIn.vInstLine12.x;
	float t1 = frm / 16;
	float t2 = frac(t1)*16;
	int i1 = ToIndex(t1);
	int i2 = ToIndex(t2/4);
	int i3 = ToIndex(frac(frm / 4) * 4);
	float2 tdmg = float2(damages1[i2][i3], damages2[i2][i3]);
	float dmg = tdmg[i1];
#endif	
	DrawDeferLight(vOut, (vsInput)vIn, transpose(mFWorld), IsVisible(dmg));
	return vOut;
}

technique TechShadowInst32
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable = false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsShadowInst32();
		PixelShader=compile ps_2_0 psShadow();
	}
}

technique TechDrawInst32
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaTestEnable=true;
		AlphaRef=iAlphaRefValue;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader=compile vs_2_0 vsDrawInst32();
		PixelShader=compile ps_2_0 psDraw();
	}
}

technique TechDeferLight32
{
	pass pass1
	{
		DEFER_LIGHT_PASS_OPTIONS
		VertexShader = compile vs_2_0 vsDeferLight32();
		PixelShader = compile ps_2_0 psDeferLight();
	}
}
