///////////////////////////////////////////////////////////////////////////////////
//      
//      . 
// World transform    
///////////////////////////////////////////////////////////////////////////////////

#include "Inc/utils.fx"
#include "Inc/warfog.fx"
#include "Inc/Shadows.fx"

///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////

float4x4	matViewProj;	// view*projection
texture		tPartTex;		//  
float4		vUpAxis[3];
float4		vRightAxis[3];
float		fTime;
float3		vCamPos;		//  
float3		vWind;			// 

#ifndef GRAPH_DETAIL_HIGH
	#define DISABLE_WIND_EFFECT
#endif

#ifndef RADEON_HD_WORKAROUND
static const float4 c_Offset[4] =	// xy - vertex offset, zw - texture coords
{
	{-0.5, -0.5, 0.0, 1.0, },	// bottom left
	{-0.5,  0.5, 0.0, 0.0, },	// upper left
	{ 0.5,  0.5, 1.0, 0.0, },	// upper right
	{ 0.5, -0.5, 1.0, 1.0, },	// bottom right
};
#else
static const float4 c_Offset[12] =	// xy - vertex offset, zw - texture coords
{
	{-0.5, -0.5, 0.0, 1.0, },	// bottom left
	{-0.5,  0.5, 0.0, 0.0, },	// upper left
	{ 0.5,  0.5, 1.0, 0.0, },	// upper right
	{ 0.5, -0.5, 1.0, 1.0, },	// bottom right

	{-0.5, -0.5, 0.0, 1.0, },	// bottom left
	{-0.5,  0.5, 0.0, 0.0, },	// upper left
	{ 0.5,  0.5, 1.0, 0.0, },	// upper right
	{ 0.5, -0.5, 1.0, 1.0, },	// bottom right
	{-0.5, -0.5, 0.0, 1.0, },	// bottom left
	{-0.5,  0.5, 0.0, 0.0, },	// upper left
	{ 0.5,  0.5, 1.0, 0.0, },	// upper right
	{ 0.5, -0.5, 1.0, 1.0, },	// bottom right
};
#endif

static const float4 c_Image[21] =	// xy - texture upper left offset, zw - texture scale
{
	{ 0.0, 0.0, 1.0, 1.0, },	// single image
	{ 0.0, 0.0, 0.5, 0.5, },	// upper left
	{ 0.5, 0.0, 0.5, 0.5, },	// upper right
	{ 0.0, 0.5, 0.5, 0.5, },	// bottom left
	{ 0.5, 0.5, 0.5, 0.5, },	// bottom right
	{ 0.00, 0.00, 0.25, 0.25, },	// 0 0
	{ 0.25, 0.00, 0.25, 0.25, },	// 1 0
	{ 0.50, 0.00, 0.25, 0.25, },	// 2 0
	{ 0.75, 0.00, 0.25, 0.25, },	// 3 0
	{ 0.00, 0.25, 0.25, 0.25, },	// 0 1
	{ 0.25, 0.25, 0.25, 0.25, },	// 1 1
	{ 0.50, 0.25, 0.25, 0.25, },	// 2 1
	{ 0.75, 0.25, 0.25, 0.25, },	// 3 1
	{ 0.00, 0.50, 0.25, 0.25, },	// 0 2
	{ 0.25, 0.50, 0.25, 0.25, },	// 1 2
	{ 0.50, 0.50, 0.25, 0.25, },	// 2 2
	{ 0.75, 0.50, 0.25, 0.25, },	// 3 2
	{ 0.00, 0.75, 0.25, 0.25, },	// 0 3
	{ 0.25, 0.75, 0.25, 0.25, },	// 1 3
	{ 0.50, 0.75, 0.25, 0.25, },	// 2 3
	{ 0.75, 0.75, 0.25, 0.25, },	// 3 3
};

///////////////////////////////////////////////////////////////////////////////////
// STREAM MAPPING
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4	vPos			:POSITION;	// 
	float4  vParams			:TEXCOORD0;	// x - plane && vertex id, y - image id, z - size, w - angle
	float4	vColor			:COLOR0;	// 
};

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sPartSampler=sampler_state
{
	Texture=(tPartTex);
	MinFilter=Linear;
	MipFilter=Point;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsOutput
{
    float4	vPos	:POSITION;	// 
    float4	vColor	:COLOR0;	// 
    float2	vTex	:TEXCOORD0;
};

#ifndef RADEON_HD_WORKAROUND
static const int2 c_Indices[12] = // x - plane index, y - offset index
{
	0, 0,
	0, 1,
	0, 2,
	0, 3,
	1, 0,
	1, 1,
	1, 2,
	1, 3,
	2, 0,
	2, 1,
	2, 2,
	2, 3,
};
#endif

vsOutput vsDefault(vsInput vIn)
{
	vsOutput vOut;

#ifndef RADEON_HD_WORKAROUND
	float sn,cs;
	sincos(vIn.vParams.w, sn, cs);
	int2 idx = c_Indices[ToIndex(vIn.vParams.x)];
	float4 coords = c_Offset[idx.y];
	float2 size = mul(coords.xy, float2x2(cs, sn, -sn, cs)) * vIn.vParams.z;
	float4 pos = vIn.vPos + vUpAxis[idx.x] * size.y + vRightAxis[idx.x] * size.x;
	vOut.vPos=mul(pos, matViewProj);
	vOut.vColor=vIn.vColor;
	float4 image = c_Image[ToIndex(vIn.vParams.y)];
	vOut.vTex = coords.zw * image.zw + image.xy;
#else
	float sn,cs;
	sincos(vIn.vParams.w, sn, cs);
	float4 coords = c_Offset[ToIndex(vIn.vParams.x)];
	float2 size = mul(coords.xy, float2x2(cs, sn, -sn, cs)) * vIn.vParams.z;
	float4 pos = vIn.vPos + vUpAxis[vIn.vParams.x / 4] * size.y + vRightAxis[vIn.vParams.x / 4] * size.x;
	vOut.vPos = mul(pos, matViewProj);
	vOut.vColor = vIn.vColor;
	float4 image = c_Image[ToIndex(vIn.vParams.y)];
	vOut.vTex = coords.zw * image.zw + image.xy;
#endif

	return vOut;
}

float4 psDefault(vsOutput vIn):COLOR0
{
	return vIn.vColor * tex2D(sPartSampler, vIn.vTex);
}

#ifndef RADEON_HD_WORKAROUND
static const float2 c_GrassData[4] =
{
	{  1.919, 0.0 },
	{  2.792, 1.5 },
	{  5.061, 1.5 },
	{ -0.349, 0.0 },
};
#else
static const float2 c_GrassData[12] =
{
	{  1.919, 0.0 },
	{  2.792, 1.5 },
	{  5.061, 1.5 },
	{ -0.349, 0.0 },

	{  1.919, 0.0 },
	{  2.792, 1.5 },
	{  5.061, 1.5 },
	{ -0.349, 0.0 },
	{  1.919, 0.0 },
	{  2.792, 1.5 },
	{  5.061, 1.5 },
	{ -0.349, 0.0 },
};
#endif

struct vsOutputGrass
{
    float4	vPos		: POSITION;
    float4	vColor		: COLOR0;
    float2	vTex		: TEXCOORD0;
    float4	vShadowParams	: TEXCOORD1;
    float3	vFogPos		: TEXCOORD2;
};

vsOutputGrass vsGrass(vsInput vIn)
{
	vsOutputGrass vOut;

#ifndef RADEON_HD_WORKAROUND
	int2 idx = c_Indices[ToIndex(vIn.vParams.x)];
	float4 coords = c_Offset[idx.y];
	float2 data = c_GrassData[idx.y];
	float sn,cs;
	sincos(vIn.vParams.w + data.x, sn, cs);
	float4 pos = vIn.vPos + float4(sn, data.y, cs, 0.0) * vIn.vParams.z;

	vOut.vPos=mul(pos, matViewProj);
	vOut.vColor=vIn.vColor;
	float4 image = c_Image[ToIndex(vIn.vParams.y)];
	vOut.vTex = coords.zw * image.zw + image.xy;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(fTime * (1.0 + frac( vIn.vPos.x + vIn.vPos.z ) ) ) * 0.03 * data.y;
#endif
	vOut.vShadowParams = Draw_GetShadowParams(pos);
	vOut.vFogPos = GetWarFogCoords(pos.xyz);
#else
	float4 coords = c_Offset[ToIndex(vIn.vParams.x)];
	float2 data = c_GrassData[ToIndex(vIn.vParams.x)];
	float sn,cs;
	sincos(vIn.vParams.w + data.x, sn, cs);
	float4 pos = vIn.vPos + float4(sn, data.y, cs, 0.0) * vIn.vParams.z;

	vOut.vPos=mul(pos, matViewProj);
	vOut.vColor=vIn.vColor;
	float4 image = c_Image[ToIndex(vIn.vParams.y)];
	vOut.vTex = coords.zw * image.zw + image.xy;
#ifndef DISABLE_WIND_EFFECT
	vOut.vTex.x += sin(fTime * (1.0 + frac( vIn.vPos.x + vIn.vPos.z ) ) ) * 0.03 * data.y;
#endif
	vOut.vShadowParams = Draw_GetShadowParams(pos);
	vOut.vFogPos = GetWarFogCoords(pos.xyz);
#endif

	return vOut;
}

float4 psGrass(vsOutputGrass vIn):COLOR0
{
	float3 shade = Draw_GetShadowSimple(vIn.vShadowParams, 1.0) * GetWarFog(vIn.vFogPos);
	return vIn.vColor * 2.0 * tex2D(sPartSampler, vIn.vTex) * float4(shade, 1.0);
}

const float cfRepeat = 200;
const float cfHalfRepeat = 100;
const float cfVRepeat = 100;

vsOutput vsSnow(vsInput vIn)
{
	vsOutput vOut;
	
#ifndef RADEON_HD_WORKAROUND
	int2 idx = c_Indices[ToIndex(vIn.vParams.x)];
	float4 coords = c_Offset[idx.y];
	float sn,cs;
	sincos(vIn.vParams.w + fTime, sn, cs);
	float2 size = mul(coords.xy, float2x2(cs, sn, -sn, cs)) * vIn.vParams.z;
	float4 vpos = vIn.vPos + float4(fmod(vWind * fTime, cfRepeat), 0);
	float3 cam_pos = vCamPos - cfHalfRepeat - vpos;
	vpos.y = cfVRepeat * frac(10 + (vpos.y - fTime * vIn.vParams.w) / cfVRepeat);
	float4 pos = vpos + vUpAxis[idx.x] * size.y + vRightAxis[idx.x] * size.x +
		cfRepeat * ceil(float4(cam_pos.x, 0, cam_pos.z, 0) / cfRepeat);
	vOut.vPos=mul(pos, matViewProj);
	vOut.vTex = coords.zw;
	vOut.vColor.rgb = 1;
	vOut.vColor.a = min(1, (vpos.y - 3) / 2);
#else
	float4 coords = c_Offset[ToIndex(vIn.vParams.x)];
	float sn,cs;
	sincos(vIn.vParams.w + fTime, sn, cs);
	float2 size = mul(coords.xy, float2x2(cs, sn, -sn, cs)) * vIn.vParams.z;
	float4 vpos = vIn.vPos + float4(fmod(vWind * fTime, cfRepeat), 0);
	float3 cam_pos = vCamPos - cfHalfRepeat - vpos;
	vpos.y = cfVRepeat * frac(10 + (vpos.y - fTime * vIn.vParams.w) / cfVRepeat);
	float4 pos = vpos + vUpAxis[vIn.vParams.x / 4] * size.y + vRightAxis[vIn.vParams.x / 4] * size.x +
		cfRepeat * ceil(float4(cam_pos.x, 0, cam_pos.z, 0) / cfRepeat);
	vOut.vPos=mul(pos, matViewProj);
	vOut.vTex = coords.zw;
	vOut.vColor.rgb = 1;
	vOut.vColor.a = min(1, (vpos.y - 3) / 2);
#endif
	return vOut;
}

technique TechAdd
{
	pass pass1
	{
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=One;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		ZEnable=true;
		ZWriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsDefault();
		PixelShader=compile ps_2_0 psDefault();
	}
}

technique TechReplace
{
	pass pass1
	{
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		ZEnable=true;
		ZWriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsDefault();
		PixelShader=compile ps_2_0 psDefault();
	}
}

technique TechGrass
{
	pass pass1
	{
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		ZEnable=true;
		ZWriteEnable=false;
		CullMode=None;
		FillMode=Solid;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsGrass();
		PixelShader=compile ps_2_0 psGrass();
	}
}

technique TechSnow
{
	pass pass1
	{
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=One;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		ZEnable=true;
		ZWriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsSnow();
		PixelShader=compile ps_2_0 psDefault();
	}
}
