///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////

float4x4	matViewProj;			//  view*proj
float4x4	matLightViewProj;		//  view*proj
float4x4	matShadowTexProj;		//  view*proj*texel_adjustment
float3		vLightDir;				//  
float4		cLightAmbient;			//  
float4		cLightDiffuse;			//  
float		fWaterScale;			//   
texture		tWaterTex;				//  
texture		tShadowMap;				//  

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sWaterSampler=sampler_state
{
	Texture=(tWaterTex);
	AddressU=Wrap;
	AddressV=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sShadowSampler=sampler_state
{
	Texture=(tShadowMap);
	AddressU=Clamp;
	AddressV=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};


///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float3	vPos		:POSITION;		//  
	float3	vNormal		:NORMAL;		// 
	float	fDepth		:TEXCOORD0;		//  
	float3  vTangent	:TEXCOORD1;		// 
};

///////////////////////////////////////////////////////////////////////////////////
// WATER SHADOW GENERATION SHADERS
///////////////////////////////////////////////////////////////////////////////////

struct vsShadowGenOutput
{
	float4	vPos:	POSITION;		//  
	float	fDepth:	TEXCOORD0;		//   
};

vsShadowGenOutput vsShadowGen(vsInput vIn)
{
	vsShadowGenOutput vOut;
//	vIn.vPos.y-=0.5;
	vOut.vPos=mul(float4(vIn.vPos,1),matLightViewProj);
	vOut.fDepth=vOut.vPos.z;
	return vOut;
}

float4 psShadowGen(vsShadowGenOutput vIn):COLOR0
{
	return vIn.fDepth;
}

technique TechShadowGen
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		StencilEnable = false;
		VertexShader=compile vs_2_0 vsShadowGen();
		PixelShader=compile ps_2_0 psShadowGen();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// RENDER TEXTURED WATER
///////////////////////////////////////////////////////////////////////////////////

struct vsPlainWaterOutput
{
	float4	vPos:		POSITION;	//  
	float2	vTex:		TEXCOORD0;	//  
	float4  vDiffuse:	COLOR0;		//  
	float2	vShadowTex:	TEXCOORD1;	//  
	float2	vDepth:		TEXCOORD2;	// x -  , y -   
};

vsPlainWaterOutput vsPlainWaterShadow(vsInput vIn)
{
	vsPlainWaterOutput vOut;
	float4 vTPos=float4(vIn.vPos,1);
	vOut.vPos=mul(vTPos,matViewProj);
	vOut.vTex=vIn.vPos.xz*fWaterScale;	
	vOut.vDiffuse=cLightDiffuse*clamp(dot(float3(0,1,0),-vLightDir),-0.3,1);
	vOut.vShadowTex=mul(vTPos,matShadowTexProj);
	vOut.vDepth=float2(vIn.fDepth,mul(vTPos,matLightViewProj).z);
	return vOut;
}

float4 psPlainWaterShadow(vsPlainWaterOutput vIn):COLOR0
{
	float4 texColor=tex2D(sWaterSampler,vIn.vTex);
	float fDepth=vIn.vDepth.y-0.005;
	float shdValue=(fDepth-tex2D(sShadowSampler,vIn.vShadowTex)>0 ? 0: 1);
	float alpha=(vIn.vDepth.x>1 ? 0.7 : vIn.vDepth.x-0.3);
	return float4((texColor*(cLightAmbient+(0.4+0.6*shdValue)*vIn.vDiffuse)).xyz,alpha);
}

technique TechDraw
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaFunc=GreaterEqual;
		StencilEnable = false;
		VertexShader=compile vs_2_0 vsPlainWaterShadow();
		PixelShader=compile ps_2_0 psPlainWaterShadow();
	}
}
