// Test for perspective shadow mapping
// Variables			// Used Technique
float4x4 matWorld;		// 1, 3
float4x4 matViewProj;	// 1, 3
texture texSM;			// 2, 3
float3 vLightDir;		// 3
float4x4 matShadow;		// 3
float4x4 matAdjust;		// 3

sampler sSMTexture = sampler_state
{
	Texture = (texSM);
	MinFilter = Point;
	MipFilter = None;
	MagFilter = Point;
};




// 1. Shadow map generation

struct SMGIn
{
	float3 vPos : POSITION;
};

struct SMGOut
{
	float4 vPos : POSITION;
	float2 vDepth: TEXCOORD0;
};

SMGOut vsSMG(SMGIn vIn)
{
	SMGOut vOut;
	float4 pos = mul(float4(vIn.vPos,1),matWorld);
	vOut.vPos = mul(pos, matViewProj);
	vOut.vDepth = vOut.vPos.zw;
	return vOut;
}

float4 psSMG(SMGOut vIn):COLOR0
{
	return vIn.vDepth.x/vIn.vDepth.y;
}

technique SMGeneration
{
	pass P0
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		VertexShader=compile vs_2_0 vsSMG();
		PixelShader=compile ps_2_0 psSMG();		
	}
}

// 2. Shadow map visualization

struct SMVIn
{
	float4 vPos:POSITION;
	float2 vTex:TEXCOORD0;
};

struct SMVOut
{
	float4 vPos:POSITION;
	float2 vTex:TEXCOORD0;
};

SMVOut vsSMV(SMVIn vIn)
{
	SMVOut vOut;
	vOut.vPos = vIn.vPos;
	vOut.vTex = vIn.vTex;
	return vOut;
}

float4 psSMV(SMVOut vIn):COLOR0
{
	return tex2D(sSMTexture, vIn.vTex);
}

technique SMVisualization
{
	pass P0
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		VertexShader=compile vs_2_0 vsSMV();
		PixelShader=compile ps_2_0 psSMV();		
	}
}

// 3. Shadow map using

struct SMUIn
{
	float3 vPos:	POSITION;
	float3 vNormal:	NORMAL;
};

struct SMUOut
{
	float4 vPos:	POSITION;
	float4 vDiffuse:COLOR0;
	float2 vSMTex:	TEXCOORD0;
	float2 vDepth:	TEXCOORD1;
};

SMUOut vsSMU(SMUIn vIn)
{
	SMUOut vOut;
	float4 pos = mul(float4(vIn.vPos,1), matWorld);
	vOut.vPos = mul(pos, matViewProj);
	vOut.vDiffuse = dot(mul(vIn.vNormal, matWorld).xyz, -vLightDir);
	vOut.vDiffuse.w = 1;
	float4 lpos = mul(pos, matShadow);
	vOut.vSMTex = mul(lpos, matAdjust);
	vOut.vDepth = lpos.zw;
	return vOut;
}

float4 psSMU(SMUOut vIn):COLOR0
{
	float d = vIn.vDepth.x/vIn.vDepth.y;
	float sd = tex2D(sSMTexture, vIn.vSMTex);
	float s = 1;
	if (d>sd) s = 0.5;
	return vIn.vDiffuse * s;
}

technique SMUsing
{
	pass P0
	{
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=true;
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		VertexShader=compile vs_2_0 vsSMU();
		PixelShader=compile ps_2_0 psSMU();		
	}
}




