// Test for shadow mapping

float4x4 mWorld;
float4x4 mViewProj;
float4x4 mLightViewProj;
float4x4 mShadowProj;
float3	 vLightDir;
float fAlphaRef;
texture tBaseTex;
texture tShadowMap;

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTex);
	MinFilter=linear;
	MagFilter=linear;
	MipFilter=none;
};

sampler sShadowMapSampler=sampler_state
{
	Texture=(tShadowMap);
	MinFilter=linear;
	MagFilter=linear;
	MipFilter=none;
};

struct vsInput
{
	float3 vPos:POSITION;
	float2 vTex:TEXCOORD0;
};

struct vsOutput
{
	float4 vPos:POSITION;
	float2 vTex:TEXCOORD0;
	float fDepth:TEXCOORD1;
};

struct vsShadowOutput
{
	float4 vPos:POSITION;
	float2 vTex:TEXCOORD0;
	float4 vTexShadow:TEXCOORD1;
	float4 vColor:COLOR0;
	float fDepth:TEXCOORD2;
};

vsOutput vsGenShadow(vsInput vIn)
{
	vsOutput vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),mWorld);
	vOut.vPos=mul(float4(vTPos,1),mLightViewProj);
	vOut.vTex=vIn.vTex;
	vOut.fDepth=vOut.vPos.z;
	return vOut;
}

vsShadowOutput vsDrawShadow(vsInput vIn)
{
	vsShadowOutput vOut;
	float3 vTPos=mul(float4(vIn.vPos,1),mWorld);
	vOut.vPos=mul(float4(vTPos,1),mViewProj);
	vOut.vTexShadow=mul(float4(vTPos,1),mShadowProj);
	vOut.vTex=vIn.vTex;
	float4 vTPosL=mul(float4(vTPos,1),mLightViewProj);
	vOut.vColor=dot(float3(0,1,0),-vLightDir);
	vOut.vColor.w=1;
	vOut.fDepth=vTPosL.z;
	return vOut;
}

PixelShader psGenShadow=asm
{
	ps_2_0
	def c0,0.5,0,0,0
	def c1,1,1,1,1
	// Incoming texture and depth
	dcl t0.xy
	dcl t1.x
	dcl_2d s0
	// sample model texture
	texld r0,t0,s0
	// subtract alpha ref from loaded alpha
	sub r1,r0.w,c0.x
	// check for transparent and kill transparent pixel
	mov r2,t1.x
	cmp r0,r1,r2,c1
	mov oC0,r0
	mov oDepth,r0.x
};

PixelShader psDrawShadow=asm
{
	ps_2_0
	def c0,0,1,0,-0.005
	def c1,0.5,0.5,0.5,1
	
	dcl v0.rgba
	dcl t0.xy
	dcl t1.xyzw
	dcl t2.x
	dcl_2d s0
	dcl_2d s1
	
	// add z-bias to depth
	add r2,t2.x,c0.w
	texld r0,t0,s0
	texld r1,t1,s1
	sub r1.r,r2,r1.r
	cmp r1,-r1.r,c0.y,c0.x

	// add lighting
	mad r1, r1, v0, c1

	mul r0,r0,r1
	// output
	mov oC0, r0
	
};

float4 psDrawShadowHLSL(vsShadowOutput vIn):COLOR0
{
	float fDepth=vIn.fDepth-0.005;
	float4 texColor=tex2D(sBaseSampler,vIn.vTex);
	float shdValue=fDepth-tex2D(sShadowMapSampler,vIn.vTexShadow);
	if (shdValue>0) shdValue=0; else shdValue=1;
	return texColor*(shdValue*vIn.vColor+float4(0.5,0.5,0.5,1));
}

technique GenerateShadow
{
	pass p1
	{
		CullMode=CCW;
		FillMode=Solid;
		Sampler[0]=(sBaseSampler);
		VertexShader=compile vs_1_1 vsGenShadow();
		PixelShaderConstant[0]=(fAlphaRef);
		PixelShader=(psGenShadow);
	}
}

technique DrawShadow
{
	pass p1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		AlphaTestEnable=true;
		AlphaRef=0xa0;
		AlphaFunc=GreaterEqual;
		Sampler[0]=(sBaseSampler);
		Sampler[1]=(sShadowMapSampler);
		VertexShader=compile vs_2_0 vsDrawShadow();
//		PixelShader=compile ps_2_0 psDrawShadowHLSL();
		PixelShader=(psDrawShadow);
	}
}


