// skinsel.fx
// Effect for selecting skinned model
// Parameters

#include "Inc/utils.fx"

///////////////////////////////////////////////////////////////////////////////////
// PARAMETERS
///////////////////////////////////////////////////////////////////////////////////

// camera
float4x4	matWorld;			// 
float4x4	matViewProj;		//  view*proj
float3		vBillBrdUp;			// Billboard up vector
float3		vBillBrdRight;		//     
texture		tBaseTexture:register(t3);		//  
// model
float4		cVisibleColor;	//   
float4		cHiddenColor;	//   
float2		vParams;			// x - FrameOffset, y - FillMode, z - AlphaRef
float4x4	arrBones[36];		//   
float4		arrFrames[80];		//  
								// x - BoneId, y - FrameDamage, z - FrameVisible

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

static const float cMeshAlphaRef = 0.25;
static const float cBBAlphaRef = 0.66;

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// Common
///////////////////////////////////////////////////////////////////////////////////

struct vsOutput
{
	float4	vPos		:POSITION;		// 
	float2	vTex		:TEXCOORD0;		//   
};

float4 Draw(vsOutput vIn, float4 vColor, float aAlphaRef)
{
	float4 tColor = tex2D(sBaseSampler, vIn.vTex);
	float a = tColor.w > aAlphaRef ? vColor.w : 0;
	return float4(vColor.xyz, a);
}

float4 psDrawVisible(vsOutput vIn) : COLOR0
{
	return Draw(vIn, cVisibleColor, cMeshAlphaRef);
}

float4 psDrawVisibleBB(vsOutput vIn) : COLOR0
{
	return Draw(vIn, cVisibleColor, cBBAlphaRef);
}

float4 psDrawHidden(vsOutput vIn) : COLOR0
{
	return Draw(vIn, cHiddenColor, cMeshAlphaRef);
}

float4 psDrawHiddenBB(vsOutput vIn) : COLOR0
{
	return Draw(vIn, cHiddenColor, cBBAlphaRef);
}

///////////////////////////////////////////////////////////////////////////////////
// Meshes
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float3	vNormal		:NORMAL;		// 
	float2	vTex		:TEXCOORD0;		//  
};

vsOutput vsDraw(vsInput vIn)
{
	vsOutput vOut;
	float4 vFrameParams = arrFrames[ToIndex(vIn.vPos.w - vParams.x)];					//    
	vOut.vTex = vIn.vTex;													//    
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);		//   
	vTPos = mul(vTPos, matWorld);
	vOut.vPos = mul(vTPos, matViewProj) * vFrameParams.z;						//   
	return vOut;
}

technique TechDraw
{
/*	pass Hidden
	{
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=false;
		ZFunc = Greater;
		VertexShader=compile vs_2_0 vsDraw();
		PixelShader=compile ps_2_0 psDrawHidden();
	}*/
	pass Visible
	{
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=false;
		ZFunc = LessEqual;
		VertexShader=compile vs_2_0 vsDraw();
		PixelShader=compile ps_2_0 psDrawVisible();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// Billboards
///////////////////////////////////////////////////////////////////////////////////

struct vsInputBB
{
	float4	vPos		:POSITION;		// xyz - , w -  
	float	fPSize		:PSIZE;			//  
	float3	vNormal		:NORMAL;		//  
	float4	vColor		:COLOR0;		//  
	float2	vTex		:TEXCOORD0;		//  
	float2	vOffset		:TEXCOORD1;		//    
};

vsOutput vsDrawBB(vsInputBB vIn)
{
	vsOutput vOut;
	float4 vFrameParams = arrFrames[ToIndex(vIn.vPos.w - vParams.x)];
	float4 vTPos = mul(float4(vIn.vPos.xyz, 1), arrBones[ToIndex(vFrameParams.x)]);
	vTPos = mul(vTPos, matWorld);
	vTPos.xyz += (vBillBrdUp * vIn.vOffset.x + vBillBrdRight * vIn.vOffset.y) * vIn.fPSize;
	// compute resulting position
	vOut.vPos = mul(vTPos, matViewProj) * vFrameParams.z;	
	vOut.vTex = vIn.vTex;
    return vOut;    	
}

technique TechDrawBB
{
/*	pass Hidden
	{
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=false;
		ZFunc = Greater;
		VertexShader=compile vs_2_0 vsDrawBB();
		PixelShader=compile ps_2_0 psDrawHiddenBB();
	}*/
	pass Visible
	{
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		CullMode=CCW;
		FillMode=Solid;
		ZEnable=true;
		ZWriteEnable=false;
		ZFunc = LessEqual;
		VertexShader=compile vs_2_0 vsDrawBB();
		PixelShader=compile ps_2_0 psDrawVisibleBB();
	}
}
