#include "Inc/WarFog.fx"
#include "Inc/Shadows.fx"
#include "Inc/PointLight.fx"

float4x4	matViewProj;		//  view*proj
float3		vLightDir;			// xyz -  
float3		cLightDiffuse;		//   
float4		cLightAmbient;		//  , w - depth bias
texture		tBaseTexture;		//  
texture		tColorTexture;
float4		vOrigin;
bool		bEnableAlpha;

sampler sBaseSampler=sampler_state
{
	Texture=(tBaseTexture);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sColorSampler = sampler_state
{
	Texture=(tColorTexture);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};


struct vsInput
{
	float4 vPos:	POSITION;
	float3 vNormal:	NORMAL;
	float2 vTex:	TEXCOORD0;
};

struct vsOutput
{
	float4	vPos		:POSITION;		// 
	float3	vDiffuse	:COLOR0;		//  
	float2	vTex		:TEXCOORD0;		//   
	float3	vFogPos		:TEXCOORD1;
	float4	vShadowParams:TEXCOORD2;		//    
	float2	vColorTex	:TEXCOORD3;
};

vsOutput vsDraw(vsInput vIn)
{
	vsOutput vOut;
	vOut.vTex = vIn.vTex;
	float fDiffuseCoeff = max(dot(vIn.vNormal, -vLightDir), -0.3);
	vOut.vDiffuse = cLightDiffuse * fDiffuseCoeff;					//    
	float4 vTPos = vIn.vPos;//float4(vIn.vPos, 1);
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vFogPos = GetWarFogCoords(vIn.vPos);
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	vOut.vColorTex = (vIn.vPos.xz - vOrigin.xy) * vOrigin.z + vOrigin.w;
	return vOut;
}

float4 psDraw(vsOutput vIn):COLOR0
{
	float4 vColor = tex2D(sBaseSampler, vIn.vTex);
	float4 color = tex2D(sColorSampler, vIn.vColorTex);
	float3 resColor = vColor.xyz * (color.xyz * 2) * (cLightAmbient.xyz + vIn.vDiffuse) * 
		Draw_GetShadowSimple(vIn.vShadowParams, cLightAmbient.w) * GetWarFog(vIn.vFogPos);
	return float4(resColor, vColor.w);
}

technique TechDraw
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=(bEnableAlpha);
		AlphaTestEnable=true;
		AlphaRef=0x01;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZWriteEnable=true;
		ZEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		StencilEnable = false;
		MultiSampleAntiAlias = true;
		VertexShader=compile vs_2_0 vsDraw();
		PixelShader=compile ps_2_0 psDraw();
	}
}

vsDeferLightOutput vsDeferLight(vsInput vIn)
{
	vsDeferLightOutput vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vTPos =vOut.vPos;
	vOut.vTNormal = vIn.vNormal;
	return vOut;
}

TECH_DEFER_LIGHT_DEFAULT
