// Surf.fx
#include "Inc/WarFog.fx"

texture tTexture;
float4x4 matViewProj;
float2 vTime;	// x = time, y = speed

sampler sTexture = sampler_state
{
	Texture = (tTexture);
	AddressU = Wrap;
	AddressV = Wrap;
	MinFilter = Linear;
	MipFilter = None;
	MagFilter = Linear;
};

struct vsInput
{
	float3 vPos : POSITION;
	float4 vTex : TEXCOORD0;		// x, y, alpha, hor alpha
	float4 vParams: TEXCOORD1;		// max tex length, speed, interval, starttime
};

struct vsOutput
{
	float4 vPos : POSITION;
	float4 vTex : TEXCOORD0;
	float4 vParams : TEXCOORD1;
	float3 vWarFog : TEXCOORD2;
};

vsOutput vsDefault(vsInput vIn)
{
	vsOutput vOut;
	vOut.vPos = mul(float4(vIn.vPos.xyz, 1), matViewProj);
	vOut.vTex = vIn.vTex;
	vOut.vParams = vIn.vParams;
	vOut.vWarFog = GetWarFogCoords(vIn.vPos);
	return vOut;
}

float4 psDefault(vsOutput vIn) : COLOR0
{
	float2 tc = float2(vIn.vTex.x - vTime.x * 0.1, 
		saturate(fmod(vIn.vTex.y + (vTime.x - vIn.vParams.w) * vIn.vParams.y, vIn.vParams.z)));
	float4 clr = tex2D(sTexture, tc);
	float s = sin(vIn.vParams.x - vIn.vTex.y);
	float4 clr2 = tex2D(sTexture, float2(-vIn.vTex.x - vTime.x * 0.1 , tc.y - s * 0.02));
	clr  = (clr + clr2) * 0.5;
	return float4(clr.xyz, clr.w * 2 * vIn.vTex.z * min(vIn.vTex.w * vIn.vTex.w * 5, 1)) * GetWarFog(vIn.vWarFog);
}

technique TechDefault
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		AlphaTestEnable=false;
		AlphaRef=0x00;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZWriteEnable=false;
		ZEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsDefault();
		PixelShader=compile ps_2_0 psDefault();
	}
}

