// diffterrain.fx
// effect for drawing diffuse terrain map with detail layers and fog


///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////
#include "Inc/WarFog.fx"
#include "Inc/Shadows.fx"

texture		tDiffuseMap;		//      
texture		tBlendMap;			//     
texture		tLayerTex0;			//  0
texture		tLayerTex1;			//  1
texture		tLayerTex2;			//  2
texture		tColorMap;
texture		tAlphaMask;
// 
float4x4	matViewProj;		//  View*Projection

float4x4	matLayer0;			//      0
float4x4	matLayer1;			//      1
float4x4	matLayer2;			//      2
// 
float2		vParams;			// x - displace map scale, y - depth bias

float3 vLightDir;			//  
float3 cLightAmbient;		// ambient 
float3 cLightDiffuse;		// diffuse 

float3		vLayers;			//     

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sDiffuseMapSampler=sampler_state
{
	Texture=(tDiffuseMap);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Point;
	MipFilter=None;
	MagFilter=Point;
};

sampler sColorSampler = sampler_state
{
	Texture=(tColorMap);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

sampler sAlphaSampler = sampler_state
{
	Texture=(tAlphaMask);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};


sampler sLayer0Sampler=sampler_state
{
	Texture=(tLayerTex0);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sLayer1Sampler=sampler_state
{
	Texture=(tLayerTex1);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sLayer2Sampler=sampler_state
{
	Texture=(tLayerTex2);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

sampler sBlendMapSampler=sampler_state
{
	Texture=(tBlendMap);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4 vPos		:POSITION;		//  ,    
	float3 vNormal	:NORMAL;		//  
	float2 vTex		:TEXCOORD0;		//    , 
									//        
};

///////////////////////////////////////////////////////////////////////////////////
// LAYER DIFFUSE BLEND WITH FOG TO DIFFUSEMAP AND SHADOW
///////////////////////////////////////////////////////////////////////////////////

struct vsOutputBlended
{
	float4 vPos			:POSITION;	//  
	float3 vColor		:COLOR0;	//   xyz-color
	float2 vDMTex		:TEXCOORD0;	//    
	float2 vLayerTex0	:TEXCOORD1;	//   
	float2 vLayerTex1	:TEXCOORD2;	//   
	float2 vLayerTex2	:TEXCOORD3;	//   
};

vsOutputBlended vsBlendLayers(vsInput vIn)
{
	vsOutputBlended vOut;
	float4 vTPos = vIn.vPos;
	float4 vLPos = float4(vIn.vPos.x, 0, vIn.vPos.z, 1);
	float fDiffuse = max(dot(vIn.vNormal, -vLightDir), -0.3);
	vOut.vColor = fDiffuse * cLightDiffuse;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vDMTex = vIn.vTex;
	vOut.vLayerTex0 = mul(vLPos, matLayer0).xz;
	vOut.vLayerTex1 = mul(vLPos, matLayer1).xz;
	vOut.vLayerTex2 = mul(vLPos, matLayer2).xz;
	return vOut;
}

float4 psBlendLayers(vsOutputBlended vIn):COLOR0
{
	float4 color = tex2D(sColorSampler, vIn.vDMTex);
	float2 DuDvCoords = vIn.vDMTex + tex2D(sBlendMapSampler, vIn.vDMTex*4) * vParams.x * color.w * 4;
	float3 DiffValue = tex2D(sDiffuseMapSampler, DuDvCoords).xxx;
	float3 vDelta = DiffValue - vLayers.xyz;
	float3 vCoeff = ((vDelta * vDelta * 2048 - 0.5) < float3(0,0,0) ? float3(1,1,1) : float3(0,0,0));
	float3x3 Layers;
	Layers[0] = tex2D(sLayer0Sampler, vIn.vLayerTex0).xyz;
	Layers[1] = tex2D(sLayer1Sampler, vIn.vLayerTex1).xyz;
	Layers[2] = tex2D(sLayer2Sampler, vIn.vLayerTex2).xyz;
	float3 LayerColor = mul(vCoeff, Layers);
	float3 TerrainColor=saturate(LayerColor * (color.xyz*2));
	TerrainColor = TerrainColor * (vIn.vColor.xyz + cLightAmbient.xyz);	
	return float4(TerrainColor, any(vCoeff));
}

technique TechLayerBlend
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=false;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=Solid;
		AlphaTestEnable=true;
		AlphaRef=0x30;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		VertexShader=compile vs_2_0 vsBlendLayers();
		PixelShader=compile ps_2_0 psBlendLayers();
	}
}

/////////////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////////////


struct vsOutputShade
{
	float4 vPos			:POSITION;	//  
	float4 vShadowParams:TEXCOORD0;
	float2 vFogPos		:TEXCOORD1;
};

vsOutputShade vsShadeTerrain(vsInput vIn)
{
	vsOutputShade vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	vOut.vFogPos = GetWarFogCoords(vTPos.xyz);
	return vOut;
}

float4 psShadeTerrain(vsOutputShade vIn):COLOR0
{
	return float4(0, 0, 0, 1 - Draw_GetShadowSimple(vIn.vShadowParams, vParams.y) * GetWarFog(vIn.vFogPos));
}

technique TechShadeTerrain
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=Solid;
		AlphaTestEnable=true;
		AlphaRef=0x00;
		AlphaFunc=Greater;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=false;
		VertexShader=compile vs_2_0 vsShadeTerrain();
		PixelShader=compile ps_2_0 psShadeTerrain();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Alpha shading
///////////////////////////////////////////////////////////////////////////////

struct vsOutputAlpha
{
	float4 vPos			:POSITION;	//  
	float3 vColor		:COLOR0;	//   xyz-color
	float2 vAlphaTex	:TEXCOORD0;
	float2 vLayerTex	:TEXCOORD1;	//   
};

vsOutputAlpha vsAlphaLayers(vsInput vIn)
{
	vsOutputAlpha vOut;
	float4 vTPos = vIn.vPos;
	float4 vLPos = float4(vIn.vPos.x, 0, vIn.vPos.z, 1);
	float fDiffuse = max(dot(vIn.vNormal, -vLightDir), -0.3);
	vOut.vColor = fDiffuse * cLightDiffuse;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vAlphaTex = vIn.vTex;
	vOut.vLayerTex = mul(vLPos, matLayer0).xz;
	return vOut;
}

float4 psAlphaLayers(vsOutputAlpha vIn):COLOR0
{
	float4 color = tex2D(sColorSampler, vIn.vAlphaTex);
	float2 DuDvCoords = vIn.vAlphaTex + tex2D(sBlendMapSampler, vIn.vAlphaTex*4) * vParams.x * color.w * 4;
	float alpha = tex2D(sAlphaSampler, DuDvCoords).x;
	float4 layer = tex2D(sLayer0Sampler, vIn.vLayerTex);
	float3 TerrainColor=saturate(layer * (color.xyz*2));
	TerrainColor = TerrainColor * (vIn.vColor.xyz + cLightAmbient.xyz);	
	return float4(TerrainColor, layer.w * (color.w > 0.9 ? (alpha > 0.5 ? 1 : 0) : alpha));
/*

	float4 color = tex2D(sColorSampler, vIn.vAlphaTex);
	float3 layer = tex2D(sLayer0Sampler, vIn.vLayerTex).xyz;
	float3 TerrainColor=saturate(layer * (color.xyz*2));
	TerrainColor = TerrainColor * (vIn.vColor.xyz + cLightAmbient.xyz);	
	return float4(TerrainColor, tex2D(sAlphaSampler, vIn.vAlphaTex).x);*/
}

technique TechLayerAlpha
{
	pass pass1
	{
		CullMode=CCW;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		FillMode=Solid;
		AlphaTestEnable=true;
		AlphaRef=0x1;
		AlphaFunc=GreaterEqual;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		VertexShader=compile vs_2_0 vsAlphaLayers();
		PixelShader=compile ps_2_0 psAlphaLayers();
	}
}



//EOF
