///////////////////////////////////////////////////////////////////////////////////
// VARIABLES
///////////////////////////////////////////////////////////////////////////////////
#include "Inc/WarFog.fx"
#include "Inc/Shadows.fx"
#include "Inc/PointLight.fx"

#ifdef GRAPH_DETAIL_LOW
	#define SIMPLE_WATER
#endif


float4x4	matViewProj;			//  view*proj
float3		vEyePos;				//  , w - DepthBias
float4		vLightDir;				//  
float3		cLightAmbient;			//  
float3		cLightDiffuse;			//  
float4		vBumpSpeed;				//      1 (xy=uv)   2(zw=vu)
float4		vParams;				// x -  
									//  -   1
									// z - coefficient of bump2 scale
texture		tCubeEnvMap;			//   
texture		tGradientTex;			//  
texture		tBumpTex;				//   

///////////////////////////////////////////////////////////////////////////////////
// SAMPLERS
///////////////////////////////////////////////////////////////////////////////////

sampler sCubeEnvSampler=sampler_state
{
	Texture=(tCubeEnvMap);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

sampler sGradientSampler=sampler_state
{
	Texture=(tGradientTex);
	AddressU=Clamp;
	AddressV=Clamp;
	AddressW=Clamp;
	MinFilter=Linear;
	MipFilter=None;
	MagFilter=Linear;
};

#ifdef SIMPLE_WATER
sampler sWaterSampler=sampler_state
{
	Texture=(tBumpTex);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Linear;
	MipFilter=Linear;
	MagFilter=Linear;
};
#endif

sampler sBumpSampler=sampler_state
{
	Texture=(tBumpTex);
	AddressU=Wrap;
	AddressV=Wrap;
	AddressW=Wrap;
	MinFilter=Point;
	MipFilter=None;
	MagFilter=Linear;
};

///////////////////////////////////////////////////////////////////////////////////
// STRUCTURES
///////////////////////////////////////////////////////////////////////////////////

struct vsInput
{
	float4	vPos		:POSITION;		//  
	float3	vNormal		:NORMAL;		// 
	float	fDepth		:TEXCOORD0;		//  
	float3  vTangent	:TEXCOORD1;		// 
};

///////////////////////////////////////////////////////////////////////////////////
// BUMP WATER
///////////////////////////////////////////////////////////////////////////////////
struct vsBumpWaterOutput
{
	float4 vPos			:POSITION;		//  
#ifdef SIMPLE_WATER
	float2 vTex			:TEXCOORD0;
#else	
	float4 vBumpTex		:TEXCOORD0;		// xy - 1 wz - 2
	float3 vView		:TEXCOORD1;		//     
	float3 vBinormal	:TEXCOORD2;		// 
	float3 vTangent		:TEXCOORD3;		// 
#endif
	float3 vNormal		:TEXCOORD4;		// 
	float4 vFogPos		:TEXCOORD5;		// xyz - fog coords, w -  
	float4 vShadowParams:TEXCOORD6;		//    
	float4 vWPos		:TEXCOORD7;		// position, w - water depth 2
};

vsBumpWaterOutput vsBumpWaterShadow(vsInput vIn)
{
	vsBumpWaterOutput vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vFogPos.xyz = GetWarFogCoords(vIn.vPos);
	vOut.vFogPos.w = vIn.fDepth * 0.05;
#ifdef SIMPLE_WATER
	vOut.vTex = vIn.vPos.xz * vParams.y;
	vOut.vNormal = vIn.vNormal;
#else	
	vOut.vBumpTex = vIn.vPos.xzzx * vParams.yyzz + frac(vBumpSpeed * vParams.x);
	vOut.vTangent = vIn.vTangent;
	vOut.vNormal = vIn.vNormal;
	vOut.vBinormal = cross(vOut.vTangent, vOut.vNormal);
	vOut.vView = vIn.vPos;
#endif
	vOut.vWPos.xyz = vTPos;
	vOut.vWPos.w = saturate(vIn.fDepth * 0.1);
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	return vOut;
}

float4 psBumpWaterShadow(vsBumpWaterOutput vIn):COLOR0
{
#ifdef SIMPLE_WATER
	float3 vNormal = normalize(vIn.vNormal);
	float fDiffuseCoeff = max(dot(vNormal, -vLightDir.xyz), -0.3);
	float shdValue = Draw_GetShadowSimple(vIn.vShadowParams, vLightDir.w);
	float cDepthAlpha = tex1D(sGradientSampler, vIn.vFogPos.w).w;
	float3 cWaterColor = tex2D(sWaterSampler, vIn.vTex);
	float3 cColor = tex2D(sWaterSampler, vIn.vTex).xyz * (cLightAmbient + cLightDiffuse * fDiffuseCoeff);
	return float4(cColor * shdValue * GetWarFog(vIn.vFogPos.xyz), cDepthAlpha);
#else
	//  -
	float4 vBump1 = tex2D(sBumpSampler, vIn.vBumpTex.xy);
	float4 vBump2 = tex2D(sBumpSampler, vIn.vBumpTex.wz);
	float4 vOffset = (vBump1 + vBump2) / 2;
	float3 vNormal = normalize(vOffset.x * vIn.vBinormal * vIn.vWPos.w + 
							   vOffset.y * vIn.vTangent * vIn.vWPos.w + vIn.vNormal);
	//  	
	float3 vView = normalize(vEyePos - vIn.vView);
	float fViewCoeff = dot(vView, vNormal);
	float fDiffuseCoeff = max(dot(vNormal, -vLightDir.xyz), -0.3);
	float3 vReflection = 2 * vNormal * fViewCoeff - vView;
	float fSpecularCoeff = pow((fDiffuseCoeff > 0 ? dot(vReflection, -vLightDir.xyz) : 0), 81);
/*	float fSpecularCoeff = (fDiffuseCoeff > 0 ? dot(vReflection, -vLightDir.xyz) : 0;
	fSpecularCoeff *= fSpecularCoeff * fSpecularCoeff;
	fSpecularCoeff *= fSpecularCoeff * fSpecularCoeff;
	fSpecularCoeff *= fSpecularCoeff * fSpecularCoeff;
	fSpecularCoeff *= fSpecularCoeff * fSpecularCoeff;*/
	float2 shdValue = Draw_GetShadow(vIn.vShadowParams, vLightDir.w);
	float3 cEnvColor = texCUBE(sCubeEnvSampler, vReflection).xyz;
	float4 cDepthColor = tex1D(sGradientSampler, vIn.vFogPos.w);
	float3 cColor = (cEnvColor * ((1 - fViewCoeff) + fViewCoeff * cDepthColor)) * 
		(cLightAmbient + shdValue.y * cLightDiffuse * fDiffuseCoeff + GetPointLightColor(vIn.vWPos, vNormal));
	cColor=saturate(cColor + shdValue.x * fSpecularCoeff * cLightDiffuse);
	return float4(cColor * GetWarFog(vIn.vFogPos.xyz), cDepthColor.w);
#endif
}

technique TechDefault
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaFunc=GreaterEqual;
		AlphaTestEnable=false;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsBumpWaterShadow();
		PixelShader=compile ps_2_0 psBumpWaterShadow();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// BUMP WATER
///////////////////////////////////////////////////////////////////////////////////
struct vsSimpleOutput
{
	float4 vPos			:POSITION;		//  
	float4 vBumpTex		:TEXCOORD0;		// xy - 1 wz - 2
	float3 vBinormal	:TEXCOORD2;		// 
	float3 vTangent		:TEXCOORD3;		// 
	float3 vNormal		:TEXCOORD4;		// 
	float2 vDepth		:TEXCOORD5;		// x -  , y -   
	float4 vShadowParams:TEXCOORD6;		//    
};

vsSimpleOutput vsSimpleWater(vsInput vIn)
{
	vsSimpleOutput vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vDepth = float2(vIn.fDepth * 0.05, saturate(vIn.fDepth * 0.1));
	vOut.vBumpTex = vIn.vPos.xzzx * vParams.yyzz + frac(vBumpSpeed * vParams.x);
	vOut.vTangent = vIn.vTangent;
	vOut.vNormal = vIn.vNormal;
	vOut.vBinormal = cross(vOut.vTangent, vOut.vNormal);
	vOut.vShadowParams = Draw_GetShadowParams(vTPos);
	return vOut;
}

float4 psSimpleWater(vsSimpleOutput vIn):COLOR0
{
	//  -
	float4 vBump1 = tex2D(sBumpSampler, vIn.vBumpTex.xy);
	float4 vBump2 = tex2D(sBumpSampler, vIn.vBumpTex.wz);
	float4 vOffset = (vBump1 + vBump2) / 2;
	float3 vNormal = normalize(vOffset.x * vIn.vBinormal * vIn.vDepth.y + 
							   vOffset.y * vIn.vTangent * vIn.vDepth.y + vIn.vNormal);
	float fDiffuseCoeff = max(dot(vNormal, -vLightDir.xyz), -0.3);
	float shdValue = Draw_GetShadowSimple(vIn.vShadowParams, vLightDir.w);
	float4 cDepthColor = tex1D(sGradientSampler, saturate(vIn.vDepth.x));
	float3 cColor = (texCUBE(sCubeEnvSampler, vIn.vNormal).xyz * 0.6 + cDepthColor * 0.4) * 
		(cLightAmbient + cLightDiffuse * fDiffuseCoeff) * shdValue;
	return float4(cColor, cDepthColor.w);
}

technique TechSimple
{
	pass pass1
	{
		CullMode=CCW;
		FillMode=Solid;
		AlphaBlendEnable=true;
		SrcBlend=SrcAlpha;
		DestBlend=InvSrcAlpha;
		AlphaFunc=GreaterEqual;
		AlphaTestEnable=false;
		PointSpriteEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		StencilEnable = false;
		MultiSampleAntiAlias = false;
		VertexShader=compile vs_2_0 vsSimpleWater();
		PixelShader=compile ps_2_0 psSimpleWater();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// DEFER LIGHT WATER
///////////////////////////////////////////////////////////////////////////////////
vsDeferLightOutput vsDeferLight(vsInput vIn)
{
	vsDeferLightOutput vOut;
	float4 vTPos = vIn.vPos;
	vOut.vPos = mul(vTPos, matViewProj);
	vOut.vTPos = vOut.vPos;
	vOut.vTNormal = vIn.vNormal;
	return vOut;
}

TECH_DEFER_LIGHT_DEFAULT