// watereffect.fx

// conversion from vsh and psh from ati radeon sdk

// VF_POS|VF_NORMAL|VF_TEXCOUNT_3|VF_TEX1_3D|VF_TEX2_3D|VF_TEX3_3D

//	vec3 position
//	vec3 normal
//  vec3 wave_height_scale
//  vec3 tex				// (u,v,w)
//	vec3 tangent
//


// constants
float4 vecVC0;			// internal constants
float4 vecVC1;			// internal constants
float4 vecVC2;			// internal constants
float4 vecVC3;			// internal constants
float4x4 matWVP;		// world*view*projection
float4 vecEye;			// camera position
float4 vecLight;		// light direction
float4 vecVC10;			// internal constants
float4 vecHeight;		// wave heights
float4 vecOffset;		// wave offsets
float4 vecSpeed;		// wave speed
float4 vecDirX;			// wave dir x
float4 vecDirY;			// wave dir y
float4 vecTimeSin;		// time and sin(time)
float4 vecDistortion;	// basetexcoord distortion
float4x4 matWorld;		// world transform
float4 vecPC0;			// internal constants
float4 vecHighlight;	// highlight color

texture tCubeTex;		// environment map cube texture
texture tGradientTex;	// water gradient 1D texture
texture tBumpTex;		// water bump texture

// samplers
sampler sBumpSampler = sampler_state
{
	Texture = (tBumpTex);
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
};

sampler sCubeTexSampler = sampler_state
{
	Texture = (tCubeTex);
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
};

sampler sGradientSampler = sampler_state
{
	Texture = (tGradientTex);
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
};

// shaders
VertexShader VShader = asm
{

	// v0    - Vertex Position
	// v3    - Vertex Normal
	// v5    - Wave Height Scale
	// v7    - Vertex Texture Data u,v 
	// v8    - Vertex Tangent (v direction)
	//
	// c0    - { 0.0,  0.5, 1.0, 2.0}
	// c1    - { 4.0, .5pi, pi, 2pi}  
	// c2    - {1, -1/3!, 1/5!, -1/7!  }  //for sin
	// c3    - {1/2!, -1/4!, 1/6!, -1/8!  }  //for cos
	// c4-7 - Composite World-View-Projection Matrix
	// c8     - ModelSpace Camera Position
	// c9     - ModelSpace Light Position
	// c10   - {fixup factor for taylor series imprecision, }(1.02, 0.1, 0, 0)
	// c11   - {waveHeight0, waveHeight1, waveHeight2, waveHeight3}  (80.0, 100.0, 5.0, 5.0) 
	// c12   - {waveOffset0, waveOffset1, waveOffset2, waveOffset3}  (0.0, 0.2, 0.0, 0.0) 
	// c13   - {waveSpeed0, waveSpeed1, waveSpeed2, waveSpeed3}   (0.2, 0.15, 0.4, 0.4)
	// c14   - {waveDirX0, waveDirX1, waveDirX2, waveDirX3}   (0.25, 0.0, -0.7, -0.8)
	// c15   - {waveDirY0, waveDirY1, waveDirY2, waveDirY3}   (0.0, 0.15, -0.7, 0.1)
	// c16   - { time, sin(time)}
	// c17   - {basetexcoord distortion x0, y0, x1, y1} (0.031, 0.04, -0.03, 0.02)
	// c18   - World Martix
	vs_1_1
	
	dcl_position v0
	dcl_normal v3
	dcl_texcoord v7
	dcl_texcoord1 v8
	dcl_texcoord2 v5
	
	mul r0, c14, v7.x       // use tex coords as inputs to sinusoidal warp 
	mad r0, c15, v7.y, r0   // use tex coords as inputs to sinusoidal warp 

	mov r1, c16.x           //time...
	mad r0, r1, c13, r0     // add scaled time to move bumps according to frequency
	add r0, r0, c12         // starting time offset
	frc r0.xy, r0           // take frac of all 4 components
	frc r1.xy, r0.zwzw      //
	mov r0.zw, r1.xyxy      //

	mul r0, r0, c10.x       // multiply by fixup factor (due to inaccuracy)
	sub r0, r0, c0.y        // subtract .5
	mul r0, r0, c1.w        // mult tex coords by 2pi  coords range from(-pi to pi)

	mul r5, r0, r0          // (wave vec)^2
	mul r1, r5, r0          // (wave vec)^3 
	mul r6, r1, r0          // (wave vec)^4 
	mul r2, r6, r0          // (wave vec)^5
	mul r7, r2, r0          // (wave vec)^6
	mul r3, r7, r0          // (wave vec)^7 
	mul r8, r3, r0          // (wave vec)^8

	mad r4, r1, c2.y, r0    //(wave vec) - ((wave vec)^3)/3! 
	mad r4, r2, c2.z, r4    //  + ((wave vec)^5)/5! 
	mad r4, r3, c2.w, r4    //  - ((wave vec)^7)/7! 

	mov r0, c0.z            //1
	mad r5, r5, c3.x ,r0    //-(wave vec)^2/2!
	mad r5, r6, c3.y, r5    //+(wave vec)^4/4!
	mad r5, r7, c3.z, r5    //-(wave vec)^6/6!
	mad r5, r8, c3.w, r5    //+(wave vec)^8/8!

//	sub r0, c0.z, v5.x      //... 1-wave scale
	mov r0, c0.z;
	mul r4, r4, r0          // scale sin 
	mul r5, r5, r0          // scale cos
	dp4 r0, r4, c11         //multiply wave heights by waves

	mul r0.xyz, v3, r0      //multiply wave magnitude at this vertex by normal 
	add r0.xyz, r0, v0      //add to position
	mov r0.w, c0.z          //homogenous component

	m4x4    oPos, r0, c4    // OutPos = ObjSpacePos * World-View-Projection Matrix

	mul     r1, r5, c11               //cos* waveheight
	dp4     r9.x, -r1, c14            //normal x offset
	dp4     r9.yzw, -r1, c15          //normal y offset and tangent offset

	mov     r5, v3                    //starting normal
	mad     r5.xy, r9, c10.y, r5      //warped normal move nx, ny according to 
									//cos*wavedir*waveeheight

	mov     r4, v8                    //tangent
	mad     r4.z, -r9.x, c10.y, r4.z  //warped tangent vector  
	 
	mov     r10, r5
	m3x3    r5, r10, c18       // transform normal
	dp3     r10.x, r5, r5
	rsq     r10.y, r10.x
	mul     r5, r5, r10.y             //normalize normal

	mov     r10, r4
	m3x3    r4, r10, c18       // transform tangent
	dp3     r10.x, r4, r4
	rsq     r10.y, r10.x
	mul     r4, r4, r10.y             //normalize tangent

	mul     r3, r4.yzxw, r5.zxyw
	mad     r3, r4.zxyw, -r5.yzxw, r3 //xprod to find binormal

	mov     r10, r0
	m4x4    r0, r10, c18       // transform vertex position

	sub     r2, c8,  r0                //view vector
	dp3     r10.x, r2, r2
	rsq     r10.y, r10.x
	mul     r2, r2, r10.y             //normalized view vector          
	
/*	add     r6, c9, r2				// HalfVector
	dp3		r10.x, r6,r6
	rsq		r10.y, r10.x
	mul		r6, r6,r10.y		// normalized HalfVector
*/
	mov     r0, c16.x
	mul     r0, r0, c17.xyxy
	frc     r0.xy, r0             //frc of incoming time
	add     r0, v7, r0            //add time to tex coords
	mov     oT0, r0               //distorted tex coord 0

	mov     r0, c16.x
	mul     r0, r0, c17.zwzw
	frc     r0.xy, r0             //frc of incoming time         
	add     r0, v7, r0            //add time to tex coords
	mov     oT1, r0.yxzw          //distorted tex coord 1

	mov     oT2, r2               //pass in view vector (worldspace)
	mov     oT3, r3               //binormal
	mov     oT4, r4               //tangent
	mov     oT5, r5               //normal
//	mov		oD0, r6					// HalfVector
};

PixelShader PShader = asm 
{
	//c0 -  Common Const (0, 0.5, 1, 0.25)
	//c1 - highlightColor (0., 0.76, 0.1, 1)
	//c2 - Light dir (1,0,0,0)
	ps_1_4
	def c3,0.2,0.2,0.2,0.2
	def c1,0.1,0.4,0.4,1
	texld r0, t0		    //bump map 0
	texld r1, t1	        //sample bump map 1
	texcrd r2.rgb, t2       //View vector
	texcrd r3.rgb, t3       //Binormal
	texcrd r4.rgb, t4       //Tangent
	texcrd r5.rgb, t5       //Normal

	add_d4  r0.xy, r0, r1  //Scaled Average of 2 bumpmaps xy offsets
	mul r1.rgb, r0.x, r3
	mad r1.rgb, r0.y, r4, r1
	mad r1.rgb, r0.z, r5, r1       //Put bumpmap normal into world space

	dp3_sat r0.rgb, r1, r2             //V.N 
	mad r2.rgb, r1, r0_x2, -r2     //R = 2N(V.N)-V

	dp3_sat r4.rgb, r1, c2			   // L.N=diff
	
	mov_sat r3, r0_x2              //2 * V.N  (sample over range of 1d map!)
	
	phase
	texld r3,r3						// water color
	dp3_sat r5.rgb,r2,c2			// reflection specular
	mul r2.rgb,r3,r4.r
	+mul r2.a,r5.r,r5.r				// spec^2
	
//	mul r2.rgb,r2,1-r0.r
//	+mul r2.a,r2.a,r2.a
	mul r2.a,r2.a,r2.a			// spec^16				
	mul r2.a,r2.a,r2.a			// spec^32				
	mad_sat r0,c1,r2.a,r2		// diff+spec
	add_sat r0,r0,c1			// diff+spec+ambient
	
	
/*	// Color with specular using halfvector	
	phase
	texld r2,r2	
	texld r3,r3
	
	mov r2,v0					// half vector
	dp3_sat r5.rgb,r1,v0		// spec
//	mul r2.rgb,r2,r2
//	+mul r2.a,r5.r,r5.r				// spec^2
	mul r2.rgb,r3,r4.r			// diffuse color
	+mul r2.a,r5.r,r5.r			// spec^4
	mul r2.rgb,r2,1-r0.r		// Frenel term
	+mul r2.a,r2.a,r2.a			// spec^8
	mul r2.a,r2.a,r2.a			// spec^16				
	mul r2.a,r2.a,r2.a			// spec^32				
	mad_sat r0,c1,r2.a,r2		// diff+spec
	add_sat r0,r0,c3			// diff+spec+ambient
//	mov r0,r2.r
*/	
/*
phase
texld r2, r2                   //cubic env map   
texld r3, r1                   //Index fresnel map using 2*V.N

//    mul r2.rgb, r2, r2             //Square the environment map
    mul r2.a, r2.g, r2.g           //use green channel of env map as specular          
//	mul r2.a,r2.g,r2.g
    mul r2.rgb, r2, 1-r0.r         //Fresnel Term 
    +mul r2.a, r2.a, r2.a          //Specular highlight ^4

    add_d4_sat r2.rgb, r2, r3_x2   //+= Water color
    +mul r2.a, r2.a, r2.a          //Specular highlight ^8
	
//	mad_sat r2.rgb,c1,r4.r,r2
	mul r2.a,r2.a,r2.a
    mad_sat r0, r2.a, c1, r2       //+= Specular highlight * highlight color
//    add_sat r0, r0,r3
    
*/
/*	phase
	texld r3,r3
	
	add r5.rgb, r2,c2		// H=(L+V)/2
	dp3_sat r5.rgb, r5, r1		// r5.r = HN
	
	mul r2.rgb,r3_x2,r4.r		// watercolor*diff
	+mul_sat r2.a,r5.r,r5.r		// spec^2
	mul r2.rgb,r2,1-r0.r
	+mul_sat r2.a,r2.a,r2.a
//	mul r3,c1,r2.a
	mad_sat r0,r3,1-r2.a,r2
//	mul r3.rgb,r3,1-r4.r
//	mov r0,r5.r
	mov r0,r2.rgb
*/
};

PixelShader PShader2 = asm
{
	ps_1_4
	def	c2,1,1,1,1
	mov r0,c2
};

// techniques
technique Water
{
	pass Pass1
	{
		Lighting = false;
		ZEnable = true;
		AlphaBlendEnable = false;
		FillMode = Solid;
		CullMode = None;
		Sampler[0] = (sBumpSampler);
		Sampler[1] = (sBumpSampler);
		Sampler[2] = (sCubeTexSampler);
		Sampler[3] = (sGradientSampler);
		VertexShaderConstant[0]=(vecVC0);
		VertexShaderConstant[1]=(vecVC1);
		VertexShaderConstant[2]=(vecVC2);
		VertexShaderConstant[3]=(vecVC3);
		VertexShaderConstant[4]=(matWVP);
		VertexShaderConstant[8]=(vecEye);
		VertexShaderConstant[9]=(vecLight);
		VertexShaderConstant[10]=(vecVC10);
		VertexShaderConstant[11]=(vecHeight);
		VertexShaderConstant[12]=(vecOffset);
		VertexShaderConstant[13]=(vecSpeed);
		VertexShaderConstant[14]=(vecDirX);
		VertexShaderConstant[15]=(vecDirY);
		VertexShaderConstant[16]=(vecTimeSin);
		VertexShaderConstant[17]=(vecDistortion);
		VertexShaderConstant[18]=(matWorld);
		PixelShaderConstant[0]=(vecPC0);
		PixelShaderConstant[1]=(vecHighlight);
		PixelShaderConstant[2]=(vecLight);
//		VertexShader = compile vs_1_1 HLSLVShader(); 
		VertexShader = (VShader);
		PixelShader = (PShader);
	}
}

